function outfname = writeSEED(tdata,format,volumeLabel,outfname)
%WRITESEED Write an ts_data object to SEED
%  By default, writes everything to one file
%  Should also write dataless SEED and miniSEED files.
%
%	format gives the format to save to: 'int16','int24'
%	volumeLabel = string label for this file (e.g. "SISMOMAR Experiment")
%
% Note: SEED doesn't handle water depth or clock drifts.

%
% W Crawford 6/2007

%% Standard beginning for ts_* functions
ts_debug(1,'Starting'); tic;          %

%% Rest of file
global SEED_FID

%% Set up universal variables
formatVersion='V2.3';	% I got my volume info from this version manual
recLen=4096;			% logical record length = 2^12=4096 (recommended);
log2RecLen=log2(recLen);
c=clock;
currentTime=ts_datetime(c(1),c(2),c(3),c(4),c(5),c(6));

%%Get data-specific values
nChans=tdata.nChannels;
[earliestTime,latestTime]=timespan(tdata);
evtime = get(tdata,'eventtime');
[evlat,evlon,evdepth,evwdepth] = get(tdata,'eventloc');
%stations=getMetaData(tdata);

%% WRITE THE FILE
SEED_FID=fopen(outfname,'ieee-be');
write_Volume_Header(formatVersion,log2RecLen,earliestTime,latestTime,currentTime,'INSU OBS Park',volumeLabel);
write_Abbreviations_And_Dictionaries(format,tdata);
write_Station_Headers(tdata,ealiestTime,latestTime);
write_DataRecords(tdata);
fclose(SEED_FID);

%% Standard end for ts_* functions
ts_debug(1,'Exiting after %.1f seconds\n',toc);

end % end of main function


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function write_Volume_Header(formatVersion,RecLen,beginTime,endTime,volumeTime,origOrg,Label)

%Start_Volume_Header_Records();
Write_Volume_ID_Blockette_5(formatVersion,RecLen,beginTime);
Write_Volume_ID_Blockette_10(formatVersion,RecLen,beginTime,endTime,volumeTime,origOrg,Label);
%Flush_Any_Remaining_Volume_Header_Records();
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function write_Abbreviations_And_Dictionaries(format,tdata)
%Start_Abbreviation_Dictionary_Header_Records();
Write_Data_Format_Dictionary_Blockette_30(format,1);

metadataArray=getMetaData(tdata);
iSC=0;
numCommentsPerStation=3;
for iStation=1:length(metadataArray),
    s=metadataArray{iStation};
    Write_Comment_Description_Blockette_31(iSC+1,'S', ...
        sprintf('%s: Clock Synced at %s',s.station,char(s.synctime)),0);
    Write_Comment_Description_Blockette_31(iSC+2,'S', ...
        sprintf('%s: Clock Drift = %g [(Inst-GPS)/elapsed]',s.station,s.clockdrift),0);
    Write_Comment_Description_Blockette_31(iSC+3,'S', ...
        sprintf('%s: Water depth = %g meters ',s.station,s.wdepth),0);
    iSC=iSC+numCommentsPerStation;
end
%Start_Generic_Abbreviation_Header_Records();
Write_Generic_Abbreviation_Blockette_33(1,'data written by AnyPlot from an ts_data object');
for i=1:nChans,
    Write_Generic_Abbreviation_Blockette_33(i+1,get(tdata,'chname',i));
end

%Start_Units_Header_Records();
Write_Units_Abbreviation_Blockette_34(1,'COUNTS','Digital Counts');
for i=1:nChans,
    Write_Units_Abbreviation_Blockette_34(i+1,get(tdata,'chunits',i),'');
end
%Flush_Any_Remaining_Dictionary_Header_Records();

% NO FIR DICTIONARY RECORDS (41)

%Start_Poles & Zeros_Dictionary_Records();
for i=1:nChans,
    Write_Response_PZ_Dictionary_Blockette_43(i,1,i+1,get(tdata,'chfilt',i));
end
% NO COEFFICIENTS DICTIONARY RECORDS (44)
% NO LIST DICTIONARY RECORDS (45)
% NO GENERIC RESPONSE DICTIONARY RECORDS (46)
% NO DECIMATION DICTIONARY RECORDS (47)

Start_Channel_Sensitivity/Gain_Dictionary_Records();
for i=1:nChans
    Write_Channel_SensGain_Dictionary_Blockette_48(i,get(tdata,'chfilt',i));
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function write_Station_Headers(tdata,earliestTime,latestTime)

metadataArray=getMetaData(tdata);
iSC=0;
for iStation=1:length(metadataArray),
    md=metadataArray{iStation};
    chanlist=getStationChans(tdata,md);
    %Start_Station_Header_Records();
    Write_Station_ID_Blockette_50(md,length(chanlist),earliestTime,numCommentsPerStation);
    Write_Station_Comment_Blockette_51(earliestTime,latestTime,iSC+1,0);
    Write_Station_Comment_Blockette_51(earliestTime,latestTime,iSC+2,0);
    Write_Station_Comment_Blockette_51(earliestTime,latestTime,iSC+3,0);
    iSC=iSC+numCommentsPerStation;
    
    for ichan=chanlist,
        Write_Channel_ID_Blockette_52(); % [52]
        Write_Response_Blockette_53(); % [53] for new, or [60] for dictionary
        % NO COEFFICIENTS BLOCKETTE [54]
        % NO DECIMATION [57]
        Write_Gain_Blockette; % [58] or [60]
        if final_sensitivity
            Write_Sensitivity_Blockette(); % [58] or [60]
        end
        for each_channel_comment,
            Write_Channel_Comment_Blockette_59(); % [59]
        end
    end
    Flush_Any_Remaining_Station_Header_Records();
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function str=makeSEEDTimeASCII(t)
if ~class(theTime,'ts_datetime'), error('Not an ts_datetime object'); end
str='';
if t.secs-floor(theTime.secs) ~=0,
    str=sprintf('%04d,%03d,%02d,%02d,%02d.%04d',t.year,t.jday, ...
        t.hours,t.minutes, ...
        floor(t.seconds),10000*(t.seconds-floor(t.seconds)));
elseif t.secs~=0,
    str=sprintf('%04d,%03d,%02d,%02d,%02d',t.year,t.jday, ...
        t.hour,t.minutes,floor(t.seconds));
elseif t.minutes~=0,
    str=sprintf('%04d,%03d,%02d,%02d',t.year,t.jday, ...
        t.hour,t.minutes);
elseif t.hour~=0,
    str=sprintf('%04d,%03d,%02d',t.year,t.jday,t.hour);
else
    str=sprintf('%04d,%03d',t.year,t.jday);
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate earliest synctime for all the channels
function [et,lt]=earliestTime(tdata)

nsamps=get(tdata,'numsamps');
et=get(tdata,'chsynctime',1);
lt=calctime(tdata,1,nsamps);

nCh=tdata.nChannels;
if nCh>1,
    for i=2:nCh
        startt=get(tdata,'chstarttime',i);
        endt=calctime(tdata,i,nsamps);
        if startt < et, et=startt; end
        if endt > lt, lt=endt; end
    end
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function metadataArray=getMetaData(tdata)
metadata=tdata.channel(1).metadata;
nMetadata=1;
metadataArray{nMetadata}=metadata;
for i=2:tdata.nChannels,
    metadata=tdata.channel(i).metadata;
    if ~strmatch(metadata,metadataArray,'exact')
        nMetadata=nMetadata+1;
        metadataArray{nMetadata}=metadata;
    end
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get the list of channel #s corresponding to the current station
function chanList=getStationChannels(tdata,station)
chanList=[];
for ich=1:tdata.nChannels
    if strcmp(station,tdata.channel(ich).metadata.station)
        chanList=[chanList ich];
    end
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Data_Format_Dictionary_Blockette_30(format,formatID)
switch format,
    case 'int24',
        description='Integer Format';
        family=0;
        keys={'M0','W3 D C2'};	% 3 bytes; order 0,1,2; 2's complement
    case 'int16',
        description='Integer Format';
        family=0;
        keys={'M0','W2 D C2'};  % 2-byte words, order 0,1; 2's complement
    otherwise
        error(['Unknown data format: ' format]);
end
nkeys=length(keys);
keystr=keys{1};
for i=2:nkeys, keystr=[keystr '~' keys{i}]; end
blength=3+4+length(description)+1+4+3+2+length(keystr);
bstr=sprintf('%03d%04d%s~04d%03d%02d%s',30,blength,description,formatID,family,nkeys,keystr);
writeblockette(bstr)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Comment_Description_Blockette_31(key,code,comment,units)
blength=3+4+4+1+length(comment)+1+3;
bstr=sprintf('%03d%04d%04d%c%s~%03d',31,blength,key,code,comment,units);
writeblockette(bstr)
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Generic_Abbreviation_Blockette_33(acode,adescription)
blength=3+4+3+length(adescription)+1;
bstr=sprintf('%03d%04d%03d%s~',33,blength,acode,adescription);
writeblockette(bstr)
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Units_Abbreviation_Blockette_34(ucode,uname,udescription)
blength=3+4+3+length(uname)+1+length(udescription)+1;
bstr=sprintf('%03d%04d%03d%s~%s~',34,blength,ucode,uname,udescription);
writeblockette(bstr)
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Response_PZ_Dictionary_Blockette_43(id,inpUnits,outUnits,filt)
respKey=id;
respName=sprintf('Chan%d PZ',id);
respType='A';	%A=Laplace transform analog response (rad/sec)
normfact=1;	% don't use normalization factor
normfreq=1;
zs=filt.zeros;
nzeros=length(zs);
zstr='';
for i=1:nzeros,
    zstr=[zstr sprintf('%12.5E%12.5E%12.5E%12.5E',real(zs(i)),imag(zs(i)),0,0)];
end
ps=filt.poles;
npoles=length(ps);
pstr='';
for i=1:npoles,
    pstr=[pstr sprintf('%12.5E%12.5E%12.5E%12.5E',real(ps(i)),imag(ps(i)),0,0)];
end

blength=3+4+4+length(respName)+1+1+3+3+12+12+3+nzeros*48+3+npoles*48;
bstr=sprintf('%03d%04d%04d%s~%c%03d%03d%12.5E%12.5E%03d%s%03d%s', ...
    43,blength,respKey,respName,respType,inpUnits,outUnits, ...
    normfact,normfreq,nzeros,zstr,npoles,nstr);
writeblockette(bstr)
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Channel_SensGain_Dictionary_Blockette_48(id,filt)
respKey=id;
respName=sprintf('Chan%d Gain');
respfreq=1;
sens=response(filt,respfreq);
nhist=0;

blength=3+4+4+length(respName)+1+12+12+2;
bstr=sprintf('%03d%04d%04d%s~%12.5E%12.5E%02d', ...
    48,blength,respKey,respName,sens,respfreq,0);
writeblockette(bstr)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Station_ID_Blockette_50(metadata,numChans,startdate,numComments)
callLetters=metadata.station
if length(callLetters)>5; callLetters=callLetters(1:5); end
lat=metadata.lat;
lon=metadata.lon;
elevation=-metadata.elev;
%%%% NO FIELD FOR WATER DEPTH!!!! %%%%
siteName='City,State/Province,Country'
networkID=1;	% lookup code 1 from blockette 33
swaporder32='3210';	% Specify that data headers will be big-endian
swaporder16='10';	% big-endian too
startDateStr=makeSEEDTimeASCII(startdate)
endDateStr=''
updateFlag='N';
networkCode=metadata.network;
if len(networkCode)~=2, networkCode='ZZ'; end

blength=3+4+5+10+11+7+4+3+length(siteName)+1+3+4+2+length(startDateStr)+1+length(endDateStr)+1+1+2;
bstr=sprintf('%03d%04d%5s%10.6f%11.6f%7.1f%04d%03d%s~%03d%04d%02d%s~%s~%c%2s', ...
    50,blength,callLetters,lat,lon,elevation, ...
    numChans,numComments,siteName,networkCode, ...
    swaporder32,swaporder16,startDateStr,endDateStr, ...
    updateFlat,networkCode);
writeblockette(bstr)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Write_Station_Comment_Blockette_51(stime,etime,key,comment)
stimestr=makeSEEDTimeASCII(stime);
etimestr=makeSEEDTimeASCII(etime);
blength=3+4+length(stimestr)+1+length(etimestr)+1+4+6;
bstr=sprintf('%03d%04d%s~%s~%04d%06d',51,blength,stimestr,etimestr,key,comment);
writeblockette(bstr);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function writeblockette(bstr)
global SEED_FID
A=sscanf(bstr,'%*3d%4d%*s');
btype=A(1); blength=A(2);
if blength~=length(bstr),
    error(sprintf('Blockette %03d: bstring has different length than calculated value',btype))
end
fwrite(SEED_FID,'%s\n',bstr);
end