function [axhand,linehand] = plot(obj,start,plotlength,varargin)
    % Plot time series data
    % [axhand,linehand]=plot(ts_data,start,length,...) plots anyplot data versus time
    %	by default, plots the entire record
    %
    %   start [default=0] is a date and time (ts_datetime or character string) or a number
    %           (seconds from the beginning of the data)
    %   length [default=0] is the number of seconds of data to plot (0 means
    %           plot to the end of data
    %
    %	... are field, value pairs
    %	'highlight' Highlights part of one or all channels
    %		value = {slist,color,channel,text,line}
    %			where
    %				slist =		ap_slist of time windows (seconds) to
    %							highlight, or equivalent text list
    %				color =		highlight color
    %				channel =	a number, or ':' for all channels [':'],
    %				text =      text in upper right corner ['']
    %				line =      Bounding box linetype [''];
    %   'scalerange' is a vector of two scalars giving the time range (in seconds)
    %           over which to calculate automatic
    %           scales (only for offset plots on one axis (overlay>0))
    %
    %	for more fields, type 'ts_data' and 'ts_data_figureproperties'
    %
    
    % Check argument values
    ts_debug(1,'Starting');
    if ~exist('start','var'); start=0; end              % Leave error checking to ts_data_channel.plot()
    if ~exist('plotlength','var'); plotlength=0; end
    
    [obj,scalerange,figInfo]= readOptions(obj,varargin{:});  
    
    start=getStart(obj,start); % set start to a ts_datetime object: if a number, set relative to earliest channel's starttime
    plotlength=getPlotLength(obj,start,plotlength); % converts strings to seconds and calculates length if plotlength==0
    
    nCh=obj.nChannels;
    if figInfo.overlay,  figInfo.nSubplots=1;
    else figInfo.nSubplots=nCh;
    end
    
    linehand = zeros(1,nCh);
    axhand=linehand;
    
    % Set time axis ticks and labels
    if isa(start,'ts_datetime')
        [figInfo.xlabel,figInfo.xticks,figInfo.xticklabels]=maketimeticks(start,start+plotlength,10);
    elseif issynchronized(obj)
        mstart=obj.channel(1).starttime+start;
        [figInfo.xlabel,figInfo.xticks,figInfo.xticklabels]=maketimeticks(mstart,mstart+plotlength,10);
    end
    
    % Plot the data (using ts_data_channel.plot())
    clf;    
    for i = 1:nCh,
        switch figInfo.overlay,
            
            %%%%%%%%%%%%%%%
            case 0,   % Each channel has its own subplot
                axes('Position',figInfo.calcplotpos(nCh,i,'allrows'));
                [axhand(i) linehand(i)] = obj.channel(i).plot(start,plotlength,figInfo,i==nCh);   % x label on bottom row only
                
                %%%%%%%%%%%%%%
            case -1,   % All channels on one axis, same scale for all
                
                if i==1,
                    [obj.channel(1).subplotprops.ymin, ...
                        obj.channel(1).subplotprops.ymax]=  setSharedScale(obj,figInfo);
                end
                [axhand(i) linehand(i)] = obj.channel(i).plot(start,plotlength,figInfo,i==nCh,'nodressing',i~=1);   % only put label grid on first plot
                drawnow, hold on
                
                %%%%%%%%%%%%%
            case -2,  % All channels on one axis, all normalized to go from -0.5 to 0.5
                plotchan=obj.channel(i);
                datarange=max(plotchan.data)-min(plotchan.data);
                plotchan.data=(plotchan.data -mean(plotchan.data)) / datarange;   % Scale to maxvalue=1;
                if i==1,
                    axes('position',figInfo.calcplotpos(1,1,'allrows'));
                    plotchan.subplotprops.ymin=-0.55; plotchan.subplotprops.ymax=0.55;
                    [axhand(i) linehand(i)] = plotchan.plot(start,plotlength,figInfo,0,'nodressing',i~=1);   % only put label grid on first plot
                else
                    [axhand(i) linehand(i)] = plotchan.plot(start,plotlength,figInfo,i==nCh,'nodressing',1);   % only put label grid on first plot
                end
                drawnow, hold on
                
                %%%%%%%%%%%%%
            otherwise 	% All channels on one axis, normalized to -0.5 to 0.5, and offset by figInfo.overlay
                plotchan=obj.channel(i);
                if ~isempty(scalerange);
                    %plotchan
                    rangedata=plotchan.cut(start+scalerange(1),scalerange(2)-scalerange(1));
                    datarange=max(rangedata.data)-min(rangedata.data);
                else
                    datarange=max(plotchan.data)-min(plotchan.data);
                end
                plotchan.data=((plotchan.data -mean(plotchan.data)) / (datarange*figInfo.overlay))  + i;
                if i==1,
                    axes('position',figInfo.calcplotpos(1,1,'allrows'));
                    plotchan.subplotprops.ymin=1-0.5/figInfo.overlay;
                    plotchan.subplotprops.ymax=(nCh+0.5/figInfo.overlay);
                    [axhand(i) linehand(i)] = plotchan.plot(start,plotlength,figInfo,i==nCh);   % only put label grid on first plot
                    % axis ij  BAD IDEA! MAKES ALL TRACES UPSIDE-DOWN
                    hold on
                else
                    [axhand(i) linehand(i)] = plotchan.plot(start,plotlength,figInfo,i==nCh,'nodressing',1);   % only put label grid on first plot
                end
        end
    end
    %axis tight
    ts_debug(1,'Ending');

    
    % ADD YLABEL IF OVERLAPPED PLOTS
    if figInfo.overlay==-1
        ylabel('Digital units');
    elseif figInfo.overlay==-2
        ylabel('Autoscaled to +- 1');
    elseif figInfo.overlay==-1
        ylabel('Channel #');
        % axis ij; BAD IDEA! MAKES ALL TRACES UPSIDE-DOWN
    end
        
    % ADD TITLE
    figInfo.addtitle('Time Series');
    set(gcf,'Name','ts_data.plot()');

    % SET CURRENT AXIS TO TOP PLOT
    axes(axhand(1));

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [obj,scalerange,figInfo]= readOptions(obj,varargin)
    figInfo=obj.figureprops;      % Make plot parameters only act in this plot
    
    % read in argument options
    i = 1;scalerange=[];
    
    % length(varargin),	% DEBUG
    while i < length(varargin),
        ich=0;
        field = varargin{i};
        if strcmp(field(1:2),'ch'),
            ich = varargin{i+1};
            value = varargin{i+2};
            i = i+3;
        else
            value = varargin{i+1};
            i = i+2;
        end
        switch field,
            case {'highlight','highlights','hilite','hilites'}
                if ~isempty(value),
                    if ~isa(value,'cell'), value={value}; end
                    hilite.slist = value{1};
                    if ischar(hilite.slist),
                        hilite.slist=ap_slist(hilite.slist);
                    end
                    if ~isa(hilite.slist,'ts_seqlist'), error('Bad first element in highlights'); end
                    lh=length(value);
                    if lh >= 2, hilite.color=value{2}; else hilite.color='y'; end
                    if lh >= 3, hilite.chan=value{3};  else hilite.chan=':'; end
                    if lh >= 4, hilite.text=value{4};  else hilite.text='';  end
                    if lh >= 5, hilite.line=value{5};  else hilite.line='';  end
                end
                ts_warning('Highlights are not implemented!')
            case 'scalerange'
                if length(value)==2 && isnumeric(value),
                    scalerange=value;
                else
                    error('scale range must be a vector of two values: start and end offsets to calculate scales');
                end
            case {'figureprops','plotinfo','figureinfo'},
                figInfo = value; 
            otherwise,
                if ich ~=0,
                    cmd=['figInfo.subplot(ich).' field '=value;'];
                else
                    cmd=['figInfo.' field '=value;'];
                end
                ts_debug(1,cmd);
                eval(cmd);
        end % switch
    end % while
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function start=getStart(obj,start)
    if isnumeric(start),
        if isnan(start), error('start is NaN!'); end
        earliest=obj.channel(1).starttime;
        if obj.nChannels>1,
            for i=2:obj.nChannels,
                if obj.channel(i).starttime<earliest, earliest=obj.channel(i).starttime; end
            end
        end
        ts_debug(1,'start=%g, converted to %s',start,char(earliest+start));
        start=earliest+start;
    elseif ischar(start)
        start=ts_datetime(start);
    end
    if ~isa(start,'ts_datetime')
        error('start is a "%s" object, wasn''t converted to ts_datetime',class(start));
    end
end

function plotlength=getPlotLength(obj,start,plotlength)
    % converts strings to seconds and calculates length if plotlength==0
    plotlength = ts_datetime.str2seconds(plotlength);
    if plotlength==0,   % Set plotlength to go to end of last data
        latest=obj.channel(1).endtime;  % time of last data in channel
        if obj.nChannels>1,
            for i=2:obj.nChannels,
                if obj.channel(i).endtime > latest,
                    latest=obj.channel(i).endtime;
                end
            end
        end
        %latest,start
        %whos
        ts_debug(1,'plotlength=%g, converted to %s',plotlength,latest-start);
        plotlength=latest-start;
    end
end

function [ymin,ymax]=setSharedScale(obj,figInfo)
    axes('position',figInfo.calcplotpos(1,1,'allrows'));
    ymin=1e10;ymax=-1e10;
    for j=1:onj.nChannels,
        if min(obj.channel(j).data) < ymin; ymin=min(obj.channel(j).data); end
        if max(obj.channel(j).data) > ymax; ymax=max(obj.channel(j).data); end
    end
end

