function fname = writeMSEED(tdata,fname,format)
    % Write a ts_data object to a miniSEED data file
    % Usage:
    % 	fName = writeMSEED(obj) creates standard miniSEED filenames using the
    %					start time & inst name
    %	fName = writeMSEED(obj,outfilename) forces the output file name
    %
    %	fName = writeMSEED(obj,outfilename,format) specifies the output format
    %		 1 = 16-bit integer (default for class 2-bit, 8-bit, int16)
    %		 3 = 32-bit integer (default for class uint16, int32)
    %		 4 = IEEE float32 (default for class single)
    %		 5 = IEEE float64 (default for all other class)
    %		10 = Steim-1 compression (D will be converted to int32)
    %		11 = Steim-2 compression (D will be converted to int32)
    %       

    % W Crawford 11/2012
    
    ts_init;	% Sets up global variables
    
    if ~exist('fname','var'), fname = makeFileName(tdata); end
    if ~exist('format','var'), format = 11; end
    ts_verbose(1,'Starting, will write file %s',fname);
    
    nc = tdata.nChannels
    net = tdata.channel(1).metadata.network
    if net = ''
        net='XX'
    for i=1:nc,     % mkmseed only writes one channel, easiest to make one
        %               file per channel, then concatenate
        chan=tdata.channel(i)
        sta = chan.metadata.station
        if sta = ''
            sta='STA'
        loc = chan.metadata.metadata
        if loc = ''
            loc='00'
        cmp = chan.metadata.component
        if cmp = ''
            cmp=sprintf('%03d',i
        temp_fname=sprintf('%s.%s.%s.%s',net,sta,loc,cmp)
        
        tdata.channel(i).writeMSEED(temp_fname,format);
    end
    
    if nc>1,    % Concatenate channel files and remove originals
        if isunix()
            unix(sprintf('cat %s.*.*.* > %s',net,fname));
            unix(sprintf('rm %s.*.*.*',net));
        elseif isdos()
            dos(sprintf('COPY %s.*.*.* %s',net,fname));
            dos(sprintf('REM %s.*.*.*',net));
        else
                error('System isn''t UNIX or DOS: Don''t know how to concatenate')
        end
    else
        if isunix()
            unix(sprintf('mv %s %s',temp_fname,fname));
        elseif isdos()
            dos(sprintf('RENAME %s %s',temp_fname,fname));
        else
                error('System isn''t UNIX or DOS: Don''t know how to move/rename')
        end
    end
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fname = makeFileName(tdata)
    % MAKE SEISAN-TYPE NAME, BASED ON FIRST TIME IN FILE AND NUMBER OF
    % CHANNELS
    
    a = getFirstLastTime(tdata);
    a=round(a); % Round first time to the nearest second
    numChans = tdata.nChannels;
    
    % PUT STATION NAME (OR NETWORK NAME IF MORE THAN ONE STATION) IN FILENAME
    instStr = tdata.channel(1).metadata.station;
    for i = 2:numChans, % If there all multiple instruments, use the network identifier
        if ~strcmp(tdata.channel(i).metadata.station,instStr), 
            instStr=tdata.channel(1).metadata.network; 
            break; 
        end
    end
    instStr = [instStr '_____']; instStr=instStr(1:5);
    
    % CREATE FILENAME
    fname = sprintf('%sM.%5s_%03d',a.char('%y-%m-%d-%H%M-%S'),instStr,numChans);
end
