function filename = writeSEGY(tdata,fname,siteName,shotInfo,delay,lengthSecs,traceFormat)
% Split a ts_data object into shots and write to a SEGY data file
% tdata.writeSEGY(fname,siteName,shotInfo,delay,length[,traceFormat]) writes a SEGY data file for each channel
%	Inputs:
%		tdata is a ts_data object
%		fname is the basename of the file to write (will append '#.sgy',
%						where # is the channel #)
%		shotInfo is a ts_shotinfo object
%		delay is the desired delay between each shot and trace start (seconds)
%		length is the length of each trace (seconds)
%		traceFormat gives the trace data format.  Valid options are:
%			1: IBM 4-byte floating point [default]
%			2: 4-byte integer
%			5: IEEE 4-byte floating point
%

% header names & values come from my old C Anyplot segy.c
%
% W Crawford 3/1999, updated for TiSKit 3/2011

ts_init;	% Sets up global variables
if ~exist('traceFormat','var'), traceFormat=1; end	% Default to IBM floating pt
switch traceFormat,
    case 1,
        disp('SEGY IBM FLOATING POINT (format 1)')
        convertToIBM=1;
        datafmt='uint32';
    case 2,
        disp('SEGY 4-byte integer (format 2)')
        convertToIBM=0;
        datafmt='int32';
    case 5,
        disp('SEGY IEEE FLOATING POINT (format 5)')
        convertToIBM=0;
        datafmt='float32';			% IEEE SEGY
    otherwise,
        error('Unknown trace format: %d',traceFormat)
end

% First get universal (channel-independent) parameters
inst.site = siteName;
trace.delay = delay;
numchans = tdata.nChannels;

for ich = 1:numchans,
    inst.samprate = tdata.channel(ich).samprate;
    trace.numsamps = lengthSecs*inst.samprate;
    chan.numsamps = tdata.channel(ich).numsamps;
    inst.lat =    tdata.channel(ich).metadata.lat;
    inst.lon =    tdata.channel(ich).metadata.lon;
    inst.depth = -tdata.channel(ich).metadata.elev;
    inst.wdepth = tdata.channel(ich).metadata.wdepth;
    inst.SN = 0;
    chnum = tdata.channel(ich).fileinfo.readchans(1);
    trace.reelseq = 0;
    ts_debug(1,'channel %d, from %s to %s',ich,...
        char(tdata.channel(1).starttime),char(tdata.channel(1).starttime+ + (chan.numsamps-1)/inst.samprate));
    filename = sprintf('%s.%d.sgy',fname,chnum);
    ts_verbose(1,'Channel %d: Creating SEGY file %s',ich,filename);
    fid = fopen(filename,'wb','b');
    chan.number = ich;  
    filt = tdata.channel(ich).metadata.response; 
    chan.gain = filt.gain; 
    chan.name=tdata.channel(1).metadata.component;
    writeEBCDICHeader(fid);
    writeReelHeader(fid,inst.samprate,trace.numsamps,traceFormat);
    numTraces=0;
    %oldStatus=-2;  madeAsterisk='*';
    for ishot = 1:length(shotInfo),
        trace.time = shotInfo(ishot).time + delay;
        [data,status,extDelay] = extractData(tdata,ich,trace.time,trace.numsamps);
        ts_debug(1,'Shottime=%s, Requested trace time = %s, actual sample delay = %5.0f ms', ...
            char(shotInfo(ishot).time),char(trace.time), 1000*extDelay);	% DEBUG
        trace.delay = delay + extDelay;	% Added 1/2006 to handling sampling interval error
        trace.time = trace.time + extDelay;	% Added 1/2006 to handling sampling interval error
        if (status == 1),		infoStr = 'beyond the end of the data';
        elseif (status == -1),	infoStr = 'before the start of the data';
        else
            trace.reelseq = trace.reelseq+1;
            trace.maxval = max(data); trace.minval = min(data);
            dist = writeTraceHeader(fid,trace,shotInfo(ishot),inst,chan);
            if convertToIBM,
                data=double(num2ibm(data));	% Stolen from SEGYMAT package
            end
            fwrite(fid,data,datafmt);	% write the data
            infoStr = sprintf('wrote SEGY  (dist = %.0f)',dist);
        end
        if abs(status)==1,
            ts_verbose(0,infoStr);
            fprintf('%d*.',shotInfo(ishot).number);
        else
            fprintf('%d.',shotInfo(ishot).number);
        end
        %oldStatus = status;
    end
    fseek(fid,3212,'bof');	% Go back to "number of traces" in reel header
    fwrite(fid,numTraces,'int16');	% 3213: Data traces per record
    fclose(fid);
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [data,status,delay]=extractData(tdata,ich,time,numsamps)
%	status = 0: got the data
%	status = 1; requested time is after the end of data
%	status = -1: requested time is before the beginning of the data
data=[];
[startAddress,delay] = calcAddress(tdata,ich,time); endAddress = startAddress+numsamps-1;
% Delay added to calcAddress 1/2006 to handling sampling interval error
if startAddress < 1, status = -1; return; end
if endAddress > get(tdata,'numsamps'); status=1; return; end
data = tdata.data(ich,startAddress:endAddress);
status = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function writeEBCDICHeader(fid)
headtxt = zeros(1,3200);
fwrite(fid,headtxt,'char');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function writeReelHeader(fid,samprate,numsamps,traceFormat)
smpint = 1e6/samprate;	% Sampling interval in usec
if smpint > 32767,
    smpint=32767; disp('***** sample interval > 32767 us, reel head is invalid (trace head ok)');
end
if numsamps >32767,
    numsamps=32767; disp('***** more than 32767 sampes, reel head is invalid (trace head ok)');
end

fwrite(fid,1,'int32');		% 3201: Job Identification Number */
fwrite(fid,0,'int32');	% 3205: Line number (one line per reel) */
fwrite(fid,1,'int32');		% 3209: Reel number */
fwrite(fid,0,'int16');	% 3213: Data traces per record (must update after writing) */
fwrite(fid,0,'int16');	% 3215: atpr: Auxiliary traces per record */
fwrite(fid,smpint,'int16');		% 3217: Sampling interval in usec for this "reel" */
fwrite(fid,smpint,'int16');		% 3219: Original sampling interval in usec */
fwrite(fid,numsamps,'int16');	% 3221: Samples per data trace (for this reel) */
fwrite(fid,numsamps,'int16');	% 3223: Original samples per data trace */
fwrite(fid,traceFormat,'int16');		% 3225: format: 1= IBM float (4 bytes)	3= fixed (2 bytes) */
%               2= fixed (4 bytes)	4= fixed+gain code (4 byte) 5=IEEE float (4 bytes)*/
fwrite(fid,0,'int16');	% 3227: CDP fold (xpcted # of traces per CDP ensemble) */
fwrite(fid,1,'int16'); 	% tsort;		/* 3229: Trace sort code:1 = as recorded (no sorting)
%		 2= CDP ensemble
%		 3= single fold continuous profile
%		 4= horizontally stacked */
fwrite(fid,1,'int16'); 	%  vsum;		/* 3231: Vertical sum code: 1= no sum, N = N sum */
fwrite(fid,1,'int16'); 	%  swfreq_s;	/* 3233: Sweep frequency at start */
fwrite(fid,1,'int16'); 	%  swfreq_e;	/* 3235: Sweep frequency at end */
fwrite(fid,1,'int16'); 	%  swlen;		/* 3237: Sweep length (ms) */
fwrite(fid,1,'int16'); 	%  swtype;	/* 3239: Sweep type code:	1= linear,	3= exponential */
%							2= parabolic,	4= other */
fwrite(fid,1,'int16'); 	%  trno;		/* 3241: Trace # of sweep channel */
fwrite(fid,1,'int16'); 	%  trtaps;	/* 3243: Sweep trace taper length at start (ms) */
fwrite(fid,1,'int16'); 	%  trtape;	/* 3245: Sweep trace taper length at end (ms) */
fwrite(fid,1,'int16'); 	%  trtype;	/* 3247: Taper type: 1= linear, 2= cos^2, 3= parabolic */
fwrite(fid,1,'int16'); 	%  trcorr;	/* 3249: Correlated data traces:	1= no,	2= yes */
fwrite(fid,1,'int16'); 	%  bingn;		/* 3251: Binary gain recoverd:	1= yes,	2= no */
fwrite(fid,1,'int16'); 	%  3253: ampmeth;	/* 3253: Amplitude recovery method:	1= none,	3= AGC */
%					2= spherical divergence	4= other */
fwrite(fid,1,'int16'); 	%  3255: distunit: 1= meters,	2=feet */
fwrite(fid,2,'int16'); 			%  3257: sign; 1= increase in pressure or upward gphone case
%					movement gives negative number on tape
% 				2= increase in pressure or upward gphone case
%					movement gives positive number on tape */
fwrite(fid,1,'int16'); 	%  vibpol;	/* 3259: Vibratory polarity:
%	Seismic sig lags pilot sig by */
%		1	=	337.5	to	22.5 */
%		2	=	22.5	to	67.5 */
%		3	=	67.5	to	112.5 */
%		4	=	112.5	to	157.5 */
%		5	=	157.5	to	202.5 */
%		6	=	202.5	to	247.5 */
%		7	=	247.5	to	292.5 */
%		8	=	292.5	to	22.5 */
restOfReelHdr = zeros(340,1);
fwrite(fid,restOfReelHdr,'char');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dist = writeTraceHeader(fid,trace,shot,inst,channel)
% shot.number, .lat, .lon,.depth
% inst.lat, .lon, .depth, .SN, .site, .samprate
% channel.number, .gain, .name
% trace.delay(secs), .reelseq, .numsamps, .time, .maxval, .minval

dist = 1852*60*sqrt((shot.lat-inst.lat)^2 + ((shot.lon-inst.lon)*cos(inst.lat*pi/180))^2);
ts_debug('shotpos=%9.2f,%9.2f, instpos=%9.2f,%9.2f, dist=%9.0f', ...
    shot.lat,shot.lon,inst.lat,inst.lon,dist);
sampint = 1e6/inst.samprate;

fwrite(fid,trace.reelseq,    'int32');	% 1:lineSeq;		/* 1:	Trace sequence number within line */
fwrite(fid,trace.reelseq,    'int32');	% 2:reelSeq;		/* 5:	Trace sequence number within reel */
fwrite(fid,shot.number,      'int32');	% 3:event_number;	/* 9:	Original field record number (event) */
fwrite(fid,channel.number,   'int32');	% 4:channel_number;/* 13:	Trace number within orig field record (channel) */
fwrite(fid,shot.number,      'int32');	% esource;	/* 17:	1:	Energy source point number */
fwrite(fid,0,                'int32');	% CDPnum;	/* 21:	CDP ensemble number */
fwrite(fid,0,                'int32');	% 7:CDPtrn;	/* 25:	Trace number within the CDP ensemble (start with 1) */
fwrite(fid,1,                'int16');	% 8:trcode;	/* 29:	Trace ident code: 1 = seismic	5= uphole
%			/*					 2 = dead		6= sweep
%			/*					 3 = dummy		7= timing
%			/*					 4 = time brk	8= water brk
%			/*		PASSCAL: =1 */
fwrite(fid,0,                'int16');	% vertsum;	/* 31:	# of vert summed traces yielding this trace */
fwrite(fid,0,                'int16');	% horzstk;	/* 33:	# of horz stacked traces yielding this trace */
fwrite(fid,1,                'int16');	% 11:use;		/* 35:	Data use: 1= production, 2= test */
fwrite(fid,dist,             'int32');	% 12:dist;		/* 37:	Distance from shotpt to receiver group (neg if opposite of shooting direction) */
scaler = 10;
fwrite(fid,-inst.depth*scaler,'int32');	% relev;	/* 41:	Elevation of receiver group (neg below sealevel) */
fwrite(fid,0,                'int32');	% selev;	/* 45:	Surface elevation at source */
fwrite(fid,shot.depth*scaler,'int32');	% sdepth;	/* 49:	Source depth below surface (always positive) */
fwrite(fid,0,                'int32');	% rdelev;	/* 53:	Datum elevation at receiver group */
fwrite(fid,0,                'int32');	% sdelev;	/* 57:	Datum elevation at source */
fwrite(fid,inst.depth*scaler,'int32');	% swdepth;	/* 61:	Water depth at source */
fwrite(fid,inst.depth*scaler,'int32');	% 19:rwdepth;	/* 65:	Water depth at receiver */
fwrite(fid,-scaler,          'int16');	% 20:zscaler;	/* 69:	Scaler to be applied to all elvs and depths in
%			/*		previous 8 variables. VAlid values are +-1, +-10
%			/*		+-100, +-1000, +-10000. Neg values divide by */
%			/*		PASSCAL: =1 */
fwrite(fid,-100,             'int16');	% 21:xyscaler;	/* 71:	Scaler to be applied to all coordinates spec'd in
%			/*		following 4 variables. Valid values are +-1, +-10
%			/*		+-100, +-1000, +-10000. Neg values divide by */
fwrite(fid,shot.lon*3600*100,'int32');	% 22:sourceX;	// 73:	Source coordinate - X (E of GM if coords are seconds)
fwrite(fid,shot.lat*3600*100,'int32');	% sourceY;	// 77:	Source coordinate - Y (N of GM if coords are seconds)
fwrite(fid,inst.lon*3600*100,'int32');	% groupX;	// 81:	Group coordinate - X
fwrite(fid,inst.lat*3600*100,'int32');	% 25:groupY;	// 85:	Group coordinate - Y
fwrite(fid,2,                'int16');	% 26:xyunits;	// 89:	Coordinate units: 1= length (m or ft), 2= secs of arc */
fwrite(fid,0,                'int16');	% wevel;	// 91:	Weathering velocity */
fwrite(fid,0,                'int16');	% subwevel;	/* 93:	Subweathering velocity */
fwrite(fid,0,                'int16');	% upsource;	/* 95:	Uphole time at source */
fwrite(fid,0,                'int16');	% 30:upgroup;	/* 97:	Uphole time at group */
fwrite(fid,0,                'int16');	% sstatic;	/* 99:	Source static correction */
fwrite(fid,0,                'int16');	% gstatic;	/* 101:	Group static correction */
fwrite(fid,0,                'int16');	% totalStatic;	/* 103:	Total static in MILLISECS applied (lower two bytes) */
fwrite(fid,0,                'int16');	% lagA;		/* 105:	Time (msec) between end of 240-byte trace ID hdr
%					& time break.  Pos if break is after header.  Time
%					break is the initiation pulse */
fwrite(fid,0,                'int16');	% lagB;		/* 107:	Time (msec) between time break & initiation
%					time of energy source */
fwrite(fid,trace.delay*1000, 'int16');	% 36:delay;	/* 109:	Time (msec) between initiation time of energy
%					source & recording of data samples (for deep
%					water work) */
fwrite(fid,0,                'int16');	% mute_s;	/* 111:	Mute time--start */
fwrite(fid,0,                'int16');	% mute_e;	/* 113:	Mute time--end */
if trace.numsamps<=32767,
    fwrite(fid,trace.numsamps,'int16');	% 39:numsamp;	/* 115:	Number of samples in this trace */
else
    fwrite(fid,32767,        'int16');	%		PASSCAL: if equal to 32767, see bytes 229-232 */
end
if sampint <= 32767,
    fwrite(fid,sampint,      'int16');	% 40:smpint;	/* 117:	Sampling interval in use for this trace */
else
    fwrite(fid,1,            'int16');	% 	PASSCAL: if equal 1, see sampint2 */
end
fwrite(fid,1,                'int16');	% 41:gaintype;	/* 119:	Gain type: 1=fixed, 2=binary, 3=floatpt. */
%/*		PASSCAL: =1 */
fwrite(fid,channel.gain,     'int16');	% gaincnst;	/* 121:	Instrument gain constant */
fwrite(fid,1,                'int16');	% initgain;	/* 123:	Instrument early or initial gain */
fwrite(fid,1,                'int16');	% corr;		/* 125:	Correlated: 1= no, 2= yes */
fwrite(fid,0,                'int16');	% 45:swfreq_s;	/* 127:	Sweep frequency at start */
fwrite(fid,0,                'int16');	% swfreq_e;	/* 129:	Sweep frequency at end */
fwrite(fid,0,                'int16');	% swlen;		/* 131:	Sweep length (ms) */
fwrite(fid,0,                'int16');	% swtype;	/* 133:	Sweep type: 1=lin, 2=parabol, 3= exp */
fwrite(fid,0,                'int16');	% trtaps;	/* 135:	Sweep trace taper length at start (ms) */
fwrite(fid,0,                'int16');	% 50:trtape;	/* 137:	Sweep trace taper length at end (ms) */
fwrite(fid,1,                'int16');	% trtype;	/* 139:	Taper type:	1= linear, 2= cos^2, 3= parabolic */
fwrite(fid,0,                'int16');	% afiltf;	/* 141:	Alias filter frequency, if used */
fwrite(fid,0,                'int16');	% afilts;	/* 143:	Alias filter slope */
fwrite(fid,0,                'int16');	% nfiltf;	/* 145:	1:	Notch filter frequency, if used */
fwrite(fid,0,                'int16');	% 55:nfilts;	/* 147:	Notch filter slope */
fwrite(fid,0,                'int16');	% lcf;		/* 149:	Low cut freq, if used */
fwrite(fid,0,                'int16');	% hcf;		/* 151:	High cut freq, if used */
fwrite(fid,0,                'int16');	% lcsl;		/* 153:	Low cut slope */
fwrite(fid,0,                'int16');	% hcsl;		/* 155:	High cut slope */
fwrite(fid,trace.time.yr,    'int16');	% 60:year;		/* 157:	year of start of trace */
fwrite(fid,trace.time.jd,    'int16');	% jday;		/* 159:	Julian day of year */
fwrite(fid,trace.time.hr,    'int16');	% hour;		/* 161:	Hour of day (24 hour clock) */
fwrite(fid,trace.time.mn,    'int16');	% minute;	/* 163:	Minute of hour */
fwrite(fid,trace.time.ss,    'int16');	% sec;		/* 165:	Second of minute */
fwrite(fid,2,                'int16');	% 65:tbase;		/* 167:	Time basis code: 1= local, 2= GMT, 3=other */
fwrite(fid,0,                'int16');	% trweight;	/* 169:	Trace weighting factor= 2^-N volts for LSB */
fwrite(fid,0,                'int16');	% ggn_roll;	/* 171:	Geophone group # of roll switch pos'n one */
fwrite(fid,0,                'int16');	% ggn_tr1;	/* 173:	Geop group # of trace #1 w/in orig field record */
fwrite(fid,0,                'int16');	% ggn_tre;	/* 175:	Geop group # of last trace w/in orig field rec */
fwrite(fid,0,                'int16');	% 70:gapsz;		/* 177:	Gap size (total # of groups dropped) */
fwrite(fid,0,                'int16');	% ovrtrvl;	// 179:	Overtravel assoc w/taper at begin/end of line
%		1= down(behind), 2 = up(ahead)
% PASCALL SEGY SECTION
temp = sprintf('%5s',inst.site); theStr = [temp(1:5) 0];
fwrite(fid,theStr,           'char');	% station_name[6];	/* 181: Station name code (5 chars + \0) */
temp = sprintf('%7d',inst.SN); theStr = [temp(1:7) 0];
fwrite(fid,theStr,           'char');	% sensor_serial[8];		/* 187: Sensor serial code (7 chars + \0) */
temp = sprintf('%3s',channel.name); theStr = [temp(1:3) 0];
fwrite(fid,theStr,           'char');	% channel_name[4];		/* 195: Channel name code (3 chars + \0) */
fwrite(fid,0,                'int16');	% 75:totalStaticHi;		/* 199: Total Static in MILISECONDS added to trace start time (high 2 bytes) */
fwrite(fid,sampint,          'int32');	% 76&77"smpint32;	/* 201:	PASCALL: Samp interval in usecs as a 32-bit integer */
fwrite(fid,1,                'int16');	% 78:format;	/* 205:	Data format flag: 0=16-bit int, 1=32-bit int */
tracems = 1000*(trace.time.ss-floor(trace.time.ss));
fwrite(fid,tracems,          'int16');	% 79:msec;		/* 207:	msecs of sec for first sample */
fwrite(fid,shot.time.yr,     'int16');	% 80:tyear;		/* 209:	Trigger time year */
fwrite(fid,shot.time.jd,     'int16');	% tjday;		/* 211:	Trigger time julian day */
fwrite(fid,shot.time.hr,     'int16');	% thour;		/* 213:	Trigger time hour */
fwrite(fid,shot.time.mn,     'int16');	% tminute;	/* 215:	Trigger time minutes */
fwrite(fid,floor(shot.time.ss),'int16'); % tsec;		/* 217:	Trigger time seconds */
fwrite(fid,1000*(shot.time.ss-floor(shot.time.ss)),'int16'); % 85: tmsec;		/* 219:	Trigger time milliseconds */
fwrite(fid,1,                'float32');	% 85: sfact;		/* 221:	Scale factor (IEEE 32 bit float)
% true amplitude = (data val)*(sfact)/gain */
fwrite(fid,inst.SN,          'int16');	% serno;		/* 225:	Instrument serial number */
fwrite(fid,0,                'int16');	% emptA;		/* 227:	Not used */
fwrite(fid,trace.numsamps,   'int32');	% numsamp32;	/* 229:	Number of samples as a 32-bit integer */
fwrite(fid,trace.maxval,     'int32');	% max;		/* 233:	Maximum value in Counts */
fwrite(fid,trace.minval,     'int32');	% min;		/* 237:	Minimum value in Counts */
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following comes from the SEGYMAT package
function b=num2ibm(x)
% num2ibm : convert IEEE 754 doubles to IBM 32 bit floating point format
%    b=num2ibm(x)
% x is a matrix of doubles
% b is a corresponding matrix of uint32
%
% The representations for NaN and inf are arbitrary
%
% See also ibm2num

%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program; if not, write to the Free Software
%    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
%
% (C) Brian Farrelly, 22 October 2001
%  mailto:Brian.Farrelly@nho.hydro.com          Norsk Hydro Research Centre
%  phone +47 55 99 68 74                 (((                  Postboks 7190
%  fax   +47 55 99 69 70                2oooS                 N-5020 Bergen
%  home  +47 55 13 78 49                HYDRO                        Norway
%

b=repmat(uint32(0),size(x));
err=zeros(size(x));

%format long

x(x> 7.236998675585915e+75)= inf;        % change big numbers to infinity
x(x<-7.236998675585915e+75)=-inf;        % 7.236998675585915e+75 is
%    ibm2num(uint32(hex2dec('7fffffff')) or
%    ibm2num(num2ibm(inf))

[F E]=log2(abs(x));

e=E/4;                         % exponent of base 16
ec=ceil(e);                    % adjust upwards to integer
p=ec+64;                       % offset exponent

f=F.*2.^(-4*(ec-e));           % correct mantissa for fractional part of exponent
f=round(f*2^24);               % convert to integer. Roundoff here can be as large as
% 0.5/2^20 when mantissa is close to 1/16 so that
% 3 bits of signifance are lost.

p(f==2^24)=p(f==2^24)+1;       % Roundoff can cause f to be 2^24 for numbers just under a
f(f==2^24)=2^20;               % power of 16, so correct for this

%format hex
psi=uint32(p*2^24);            % put exponent in first byte of psi.
phi=uint32(f);                 % put mantissa into last 3 bytes of phi

% make bit representation

b=bitor(psi,phi);                        % exponent and mantissa
b(x<0)=bitset(b(x<0),32);                % sign bit
%format long

% special cases

b(x==0)          =uint32(0)                  ;         %  bias is incorrect for zero
b(isnan(x))      =uint32(hex2dec('7fffffff'));         %  7.237005145973116e+75 in IBM format
b(isinf(x) & x>0)=uint32(hex2dec('7ffffff0'));         %  7.236998675585915e+75    ,,
b(isinf(x) & x<0)=uint32(hex2dec('fffffff0'));         % -7.236998675585915e+75    ,,
% Note that NaN > inf in IBM format

% check bit representation for normal cases

checkx=ibm2num(b);                      % note that use of base 16 in IBM format
z=x==0;                                 % can lead to a loss of 3 bits of precision
err(z)=0;                               % compared with an IEEE single.
q=(checkx(~z)-x(~z))./x(~z);
err(~z) = abs(q) > 5e-7;                % this is almost reached with numbers
% of the form 16^n + 0.5*16^(n-5) where
% the mantissa is 100001 hex. Roundoff
% error is then 0.5/16^5=0.5/2^20=4.7684e-7


if any(err)
    warning('Conversion error in num2ibm for the following:')
    disp(x(logical(err)))
end
end



function x=ibm2num(b)
% ibm2num : convert IBM 32 bit floating point format to doubles
%    x=num2ibm(b)
% b is a matrix of uint32
% x is a corresponding matrix of doubles
x=repmat(NaN,size(b));

sign=bitget(b,32);                            % get sign from first bit
sign=double(sign);

% format hex
exp=bitand(b,uint32(hex2dec('7f000000')));    % get exponent from first byte, last 7 bits
exp=bitshift(exp,-24);
%format long
exp=double(exp)- 64;                          % remove bias from exponent

%format hex
frac=bitand(b,uint32(hex2dec('00ffffff')));   % get mantissa from last 3 bytes
%format long
frac=double(frac);
frac=frac/2^24;

x=(1-2*sign).*16.^exp .* frac;

err = frac==0 & (exp~=-64 | sign~=0);         % bias removal is incorrect for zero
if any(err)
    % TMH 19/06/2003
    fprintf('WARNING : %s Invalid zero input --> Sure data are IBM FLOAT formatted ?',mfilename)
    return;
    %warning('Invalid zero input in ibm2num for the following:')
    % format hex; disp(b(err)); format
end

err = frac~=0 & (frac<1/16 | frac>=1);
if any(err)
    % TMH 19/06/2003
    fprintf('WARNING : %s Invalid mantissa input --> Sure data are IBM FLOAT formatted ?',mfilename)
    return;
    % warning('Invalid mantissa input in ibm2num for the following:')
    % format hex; disp(b(err)); format
end
end





