function fname = writeSEISAN(tdata,fname)
% Write a ts_data object to a SEISAN data file
% Usage:
% 	fName = writeSEISAN(obj) creates standard SEISAN filenames using the
%					start time & inst name
%	fName = writeSEISAN(obj,outfilename) forces the output file name
%	
% Note: all channels must start within 1/2 sample of each other

% header names & values come from the SEISAN Users Guide (v8.0)
%
% W Crawford 6/2005

ts_init;	% Sets up global variables

numChans = tdata.nChannels;

if ~exist('fname','var'), fname = makeFileName(tdata); end
ts_verbose(1,'Starting, will write file %s',fname);
		
fid=fopen(fname,'wb');
ts_debug(1,'Writing main header');
writeMHead(fid,tdata);

for i=1:numChans,
    chan=tdata.channel(i);
    numSamps = chan.nSamples;
	ts_debug(1,'Writing channel %d',i); 
	writeCHead(fid,chan);
	writeFHeader(fid,4*numSamps);
	fwrite(fid,chan.data,'int32');
	writeFTail(fid,4*numSamps);
end
ts_debug(1,'Closing file and ending'); 
fclose(fid);
return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function writeMHead(fid,tdata)

maxSEISAN=600000;	% Max # of samples/chan in standard seisan
numChans = tdata.nChannels;
[a,b]=getFirstLastTime(tdata);

% LINE 1
networkName=tdata.channel(1).metadata.network;     % ASSUMES ALL IN SAME NETWORK
numStations=numChans;	% ??? or number of instruments???
if a.year<2000, centCode=0; yr=a.year-1900;
else centCode=1; yr=a.year-2000;
end
myStr=sprintf(' %-29s%3d%1d%02d %s %9.3f%11s', ...
	networkName,numStations,centCode,yr,a.char('%j %m %d %H %M %S.%F'),b-a,'');
writeFHeader(fid,80);
fwrite(fid,myStr,'80*char');
writeFTail(fid,80);

% LINE 2: 80 character empty string
writeFHeader(fid,80);
fwrite(fid,blanks(80),'80*char');
writeFTail(fid,80);

% LINES 3-12+: Channel information, 3 channels per line, at least 10 lines
% (can have blanks)
if numChans<=30,
    nlines=10;
else
    nlines=ceil(numchans/3);
end
chanStrs=cell(3*nlines,1);

% Fill channel strings (26 characters each)
for i=1:length(chanStrs); chanStrs{i}=blanks(26); end
for i=1:numChans,
    chan=tdata.channel(i);
	statCode= [chan.metadata.station      blanks(4)];   % Make sure it's at least 4 chars long
	component=[chan.metadata.component blanks(4)];   % Make sure it's at least 4 chars long
	stime=chan.starttime;
    sampInt = 1/chan.samprate;
    numSamps = chan.nSamples;
    numSecs = (numSamps-1)*sampInt;
    if numSamps > 999999,	ts_warning('%d SAMPLES: SEISAN FILE FORMAT ONLY ALLOWS 6 DIGITS (UP TO 999999)',numSamps);
    elseif numSamps > maxSEISAN, ts_warning('%d samples: standard SEISAN is limited to %d',numSamps,maxSEISAN); end
	chanStrs{i}=sprintf(' %4s%4s%c%7.2f %8.2f',statCode(1:4),component(1:4),statCode(5),stime-a,numSecs);
end

% Write channel strings to file (3 per line)
for i=0:nlines -1,
	writeFHeader(fid,80);
	for j=1:3,
		fwrite(fid,chanStrs{3*i+j},'26*char');
	end
	fwrite(fid,blanks(2),'2*char');
	writeFTail(fid,80);
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function writeCHead(fid,chan)

% Set up channel values to save
statCode =  [chan.metadata.station '     '];	% Force to at least 5 chars long
component = [chan.metadata.component '    '];		% Force to at least 4 chars long
timeQuality=' ';	% blank=time ok, 'E' uncertain time
samprate = chan.samprate;
numsamps = chan.nSamples;
lat = chan.metadata.lat;
lon = chan.metadata.lon;
elev= chan.metadata.elev;
a=chan.starttime;
if a.year<2000, centCode=0; yr=a.year-1900;
else centCode=1; yr=a.year-2000; end
gainFactStr=' ';	% Blank: No gain factor, G: Gain factor in column 148 to 1
dataFormat=4;		% Data in 4-byte integers
respTypeStr='P ';	% Response given as poles-zeros (option 1 in Seisan format description)
myStr=sprintf('%5s%4s', statCode(1:5),component(1:4));
myStr=[myStr sprintf('%1d%02d %s%c%6.3f', ...
	centCode,yr,a.char('%j %m %d %H %M'),timeQuality,a.second)];
myStr=[myStr sprintf(' %7.2f%7d %8.4f %9.4f %5.0f%c%1d%2s ', ...
	samprate,numsamps,lat,lon,elev,gainFactStr,dataFormat,respTypeStr)];

%  Start writing
writeFHeader(fid,1040);
fwrite(fid,myStr,'char');	% addresses 1-80
fwrite(fid,blanks(80),'char');		% addresses 81-160

% Now seismometer poles and zeros. 
% Note: response is assumed to be in DISPLACEMENT (counts/m)
filt = chan.metadata.response;
poles=filt.poles; zeros=filt.zeros; gain=filt.gain;
if isempty(gain), gain=1; end
nP = length(poles); nZ=length(zeros);
myStr = sprintf(' %5d%5d%11.4g',nP,nZ,gain);
fwrite(fid,myStr,'22*char');	% addresses 161-182

endAddr=182;
% Addresses 183-240 are reserved for poles only
for i = 1:nP,
	%real(poles(i)), imag(poles(i))
	myStr = sprintf('%11.4g',real(poles(i)));
	fwrite(fid,myStr,'11*char');
	endAddr=endAddr+11;
	if rem(endAddr,80)>=77,	% write blank characters to fill in line
		for j=1:80-rem(endAddr,80),
			fwrite(fid,' ','char');
			endAddr=endAddr+1;
		end
	end
	myStr = sprintf('%11.4g',imag(poles(i)));
	fwrite(fid,myStr,'11*char');
	endAddr=endAddr+11;
	if rem(endAddr,80)>=77,	% write blank characters to fill in line
		for j=1:80-rem(endAddr,80),
			fwrite(fid,' ','char');
			endAddr=endAddr+1;
		end
	end
end
for i = 1:nZ,
	myStr = sprintf('%11.4g',real(zeros(i)));
	fwrite(fid,myStr,'11*char');
	endAddr=endAddr+11;
	if rem(endAddr,80)>=77,	% write blank characters to fill in line
		for j=1:80-rem(endAddr,80),
			fwrite(fid,' ','char');
			endAddr=endAddr+1;
		end
	end
	myStr = sprintf('%11.4g',imag(zeros(i)));
	fwrite(fid,myStr,'11*char');
	endAddr=endAddr+11;
	if rem(endAddr,80)>=77,	% write blank characters to fill in line
		for j=1:80-rem(endAddr,80),
			fwrite(fid,' ','char');
			endAddr=endAddr+1;
		end
	end
end

if endAddr > 1040, error('Oops! more than 37 pole-zeros wrote: overran header!');
elseif endAddr < 1040,
	fwrite(fid,blanks(1040-endAddr),'char');
	%endAddr = endAddr+1040-endAddr;
end

writeFTail(fid,1040);
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [a,b] = getFirstLastTime(tdata)
numChans = tdata.nChannels;
a = tdata.channel(1).starttime;
b = a;
for i = 1:numChans,
    chan=tdata.channel(i);
    numSamps = chan.nSamples;
    sampInt = 1/chan.samprate;
    numSecs = (numSamps-1)*sampInt;
	stime=chan.starttime;
	if stime < a,
		a = stime;	% a should be earliest time in the file
	end 
	if stime+numSecs > b,
		b = stime+numSecs;
	end
end
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write the equivalent of a FORTRAN unformatted record separator
function writeFHeader(fid,num)
fwrite(fid,num,'int32');
return;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write the equivalent of a FORTRAN unformatted record separator
function writeFTail(fid,num)
fwrite(fid,num,'int32');
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fname = makeFileName(tdata)

a = getFirstLastTime(tdata);
a=round(a); % Round first time to the nearest second
numChans = tdata.nChannels;

% PUT STATION NAME (OR NETWORK NAME IF MORE THAN ONE STATION) IN FILENAME
instStr = tdata.channel(1).metadata.station;
for i = 2:numChans, % If there all multiple instruments, use the network identifier
	if ~strcmp(tdata.channel(i).metadata.station,instStr), instStr=tdata.channel(1).metadata.network; break; end
end
instStr = [instStr '_____']; instStr=instStr(1:5);

% CREATE FILENAME
fname = sprintf('%sS.%5s_%03d',a.char('%y-%m-%d-%H%M-%S'),instStr,numChans);
