function obj = cut(obj,start,len,startcriter,endcriter)
    % Extract data within a given time period
    % obj = cut(obj,start,length) creates a new ts_data_channel
    % 		object containing only the data from start of a given length
    %
    %	start, end:    start and end of data to cut (see below for types)
    %	startcriteria: '>=' or '>': whether to allow the specified time, or just after  ['>=']
    %   endcriteria:   '<=' or '<': whether to allow the specified time, or just before ['<']
    %
    %   Allowed types for start and end:
    %		start     length      Interpretation
    %	1:	numeric   numeric     start is time in seconds from data start, length is in seconds
    %	2:	ts_data   numeric     start is absolute time, length is in seconds
    %	3:	ts_data   ts_data     start is absolute time, "length" is absolute end time
    %
    %   if length=0, will output from 'start' to the end of the data
    %
    %	For case 3, the data will be cut at the last sample BEFORE the end time
    %
    %
    
    if nargout ==0, error('Must provide an output argument'); end
    
    if isa(start,'numeric') && isa(len,'numeric')
        if start==0 && len==0
            ts_verbose(2,'start=len=0: Not cutting data');
            return;
        end
    end
    if ~exist('startcriter','var'), startcriter='>=';
    else
        switch startcriter,
            case {'>','>='}
            otherwise
                error('Unknown start criteria ''%s'', use ''>='' or ''>''',startcriter);
        end
    end
    if ~exist('endcriter','var'),     endcriter='<';
    else
        switch endcriter,
            case {'<','<='}
            otherwise
                error('Unknown end criteria ''%s'', use ''<='' or ''<''',endcriter);
        end
    end
    
    % start, len   % DEBUG
    [istart, iend] = getAddress(obj,start,len,startcriter,endcriter);
    
    % VERBOSE AND DEBUGGING INFORMATION
    if isa(start,'numeric')
        if isa(len,'numeric'),
            if len==0, ts_verbose(1,'Cutting data from %s%g seconds to end: %d samples',startcriter,start,iend-istart+1);
            else ts_verbose(2,'Cutting data from %s%g to %s%g seconds: %d samples',startcriter,start,endcriter,start+len,iend-istart+1);
            end
        else ts_verbose(2,'Cutting data from %s%g seconds to %s%s: %d samples',startcriter,start,endcriter,char(len),iend-istart+1);
        end
    else
        if isa(len,'numeric'),
            if len==0, ts_verbose(2,'Cutting data from %s%s to end: %d samples',startcriter,char(start),endcriter,iend-istart+1);
            else ts_verbose(2,'Cutting data from %s%s to %s%s: %d samples',startcriter,char(start),endcriter,char(start+len),iend-istart+1);
            end
        else ts_verbose(2,'Cutting data from %s%s to %s%s: %d samples',startcriter,char(start),endcriter,char(len),iend-istart+1);
        end
    end
    ts_debug(2,'Cut data from address %d to %d', istart,iend);
    
    % Cut the data
    fprintf('%d to %d (of 1 to %d)\n',istart,iend, length(obj.data))
    ts_debug(1,'istart=%d, iend=%d length(data)=%d\n',istart,iend, length(obj.data))
    if ( istart<=0 || iend > length(obj.data) )
        obj.data=nan;
        if istart <= 0
            ts_warning('istart is <=0');
        end
        if iend > length(obj.data)
            ts_warning('istart is <=0');
        end
    else
        obj.data = obj.data(istart:iend);
        obj.starttime= obj.calcTime(istart);
    end
end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [istart, iend] = getAddress(obj,startPt,endPt,startCriter,endCriter)
    % Calculate address corresponding to the startPt and endPt
    % possible types for startPt and endPt are:
    %   startPt                 endPt
    %  ==========             =================
    %   datetime                datetime
    %   datetime                numeric (seconds since startPt)
    %   numeric                 numeric ("         ""   ""    )
    % where "datetime is a ts_datetime object or a character time string)
    
    % Convert chars to ts_datetime
    if isa(startPt,'char'), startPt = ts_datetime(startPt); end
    if isa(endPt,'char'),     endPt = ts_datetime(endPt); end
    
    if isa(startPt,'ts_datetime'),
        if ~isa(endPt,'ts_datetime') && ~isnumeric(endPt),
            error('endPt is neither a ts_datetime nor a numeric value');
        end
    elseif isnumeric(startPt)
        if ~isnumeric(endPt),
            error('startPt is numeric, but endPt is not');
        end
    else
        error('startPt is neither a ts_datetime nor a numeric value');
    end
    
    % Calculate start address
    istart = obj.calcAddress(startPt,startCriter);
    
    %startPt,endPt   % DEBUG
    
    if isa(endPt,'ts_datetime'),
        if ~isa(startPt,'ts_datetime'),
            error('you specified the length in ts_datetime without a start in ts_datetime!')
        else
            iend=obj.calcAddress(endPt,endCriter);
        end
    elseif isa(endPt,'numeric')	% endPt is a number, treat as length in seconds
        if endPt==0,    % want data all the way to the end
            iend=obj.nSamples;
        else
            f_end=istart+endPt*obj.samprate;
            switch endCriter,
                case '<',
                    iend = ceil(f_end-1);
                case '<=',
                    iend = floor(f_end);
                otherwise
                    error('Unknown end criteria ''%s'', use ''<='' or ''<''',endcriter);
            end
            %startPt+endPt       % DEBUG
            %iend = istart+ceil(endPt*obj.samprate)-1;   % Always same length
        end
    else
        error('end must be ''ts_datetime'' or numeric, not ''%s''',class(endPt));
    end;
    
    %istart,iend     % DEBUG
    % Check for errors
    if istart > iend,
        if isa(startPt,'ts_datetime') && isa(endPt,'ts_datetime'),
            error('Start time (%s) is after end time (%s)', ...
                char(startPt),char(endPt))
        else
            error('Start address (%d) is greater than end address (%d)', ...
                istart,iend);
        end
    end
    
    % avoid the starttime being within the hour or day before the rest of the data
    if istart==0,
        istart=1;
        ts_verbose(1,'Shifted start address from 0 to 1');
    end  
    
    if istart < 1,
        if iend>0, % Channel contains data within cut range
            if isa(startPt,'ts_datetime'),
                if istart<0 % only warn if there is more than 1 sample offset
                    ts_warning('Requested start time (%s) is before channel start (%s), shifted to start', ...
                        char(startPt),char(obj.starttime) );
                end
            else
                ts_warning('Start address (%d) is < 1, shifted to 1',istart);
            end
        else % No data in cut range
            ts_warning('No data in cut range');
            iend=0;
            istart=0;
        end
    end
    
    nS = obj.nSamples;
    if iend > nS,	% Save what you can if possible, give error if not
        sr = obj.samprate;
        if istart<nS,
            if isa(endPt,'ts_datetime'),
                ts_warning('End time (%s) is after channel end (%s), shortening to fit data', ...
                    char(endPt),char(obj.starttime)+nS/sr )
            else
                if iend>nS+1    % Don't give a warning on a 1-sample offset
                    ts_warning('End address (%d) > data length (%d), shortening to fit data',iend,nS);
                end
            end
            iend=nS;
        else
            if isa(endPt,'ts_datetime'),
                error('Start and end times (%s) are after channel end (%s)', ...
                    char(endPt),char(obj.starttime')+nS/sr )
            else
                error('Start (%d) and end addresses (%d) > data length (%d)',istart,iend,nS);
            end
        end
    end
end
    
