function [obj,leftover] = resample(obj,newsamprate)
    %Change data sampling rate
    %    obj = resample(obj,newsamprate)
    %    [obj,leftover] = resample(obj,newsamprate)
    % Inputs [default]:
    %  obj              A ts_data_channel object
    %  newsamprate        Desired sampling frequency.  Data are interpolated at the new
    %                     frequency.  If the new sampling frequency is lower than the old,
    %                     the data are 2-pole causal lowpass filtered at newsamprate/2
    %                     before interpolating
    %           -or-
    %                     A string giving the number of original samples to average
    %                       over.  The form is
    %                       '/N' for decimation by N.  Each new sample is an average
    %                           of the N samples it replaces.  Warns if the original
    %                           number of samples is not evenly divisable by N
    %                       '*N' for Lanczos oversampling (windowed normalized sinc
    %                           convolution)
    %
    % Outputs:
    %	obj		Resampled data (ts_data_channel object)
    %	leftover	Any leftover data at the original sampling rate (ts_data_channel)
    %
    % Examples:
    %      obj = resample(obj,3.5);      % Resample at 3.5 Hz
    %      obj = obj.resample(3.5);      % Ditto
    %      [obj,lo] = obj.resample('/4');  % Resample by averaging clumps of 4
    %                                        samples. If the original number of
    %                                        samples isn't divisible by 4,
    %                                        returns the last (unused) samples in "lo"
    %      obj= obj.resample('*4');      % Resample at 4 times the original
    %                                      frequency, using Lanczos oversampling
    %
    
    % W Crawford 11/99.  Component renaming added 01/2011
    
    if nargout ==0, error('Must provide an output argument'); end
    
    leftover = ts_data_channel();
    oldsamprate = obj.samprate;
    nS_old = obj.nSamples;
    if isa(newsamprate,'char'),	% Undersample or oversample
        switch newsamprate(1)
            case '/' % Divide by X
                divisor = sscanf(newsamprate,'/%d');
                if length(divisor)~=1, error('Bad divisor string'); end
                newsamprate = oldsamprate/divisor;
                ts_verbose(0,'Resampling ts_data every %d samples (from %g to %g Hz)\n', ...
                    divisor,oldsamprate,newsamprate);
                remainder = rem(nS_old,divisor);
                if remainder ~=0,
                    ts_warning('Divisor is not a denominator of the input data, cut off last %d input samples (returned as ''remainder'')', remainder);
                    leftover = obj.cut_addr(nS_old-remainder+1,nS_old);
                end
                timeoffset = ((divisor-1)/2)/oldsamprate;
                data = obj.data(1:nS_old-remainder);
                data = reshape(data,divisor,(nS_old-remainder)/divisor);
                newdata = mean(data);
            case '*';
                multiplier = sscanf(newsamprate,'*%d');
                if length(multiplier)~=1, error('Bad multiplier string'); end
                newdata=SincResample(obj.data, obj.nSamples*multiplier);
                newsamprate=oldsamprate*multiplier;
                timeoffset=0;   %???????????
            otherwise
                error('Unknown resample operator: %s',newsamprate);
        end
    else
        ts_verbose(1,'Resampling ts_data from %g to %g Hz',oldsamprate,newsamprate);
        if newsamprate < oldsamprate,	% Need to antialias filter
            obj = obj.filter('low',2,newsamprate/2);
        end
        data = obj.data;
        oldtimes = (1:nS_old)/oldsamprate;
        newtimes = 1/newsamprate:1/newsamprate:max(oldtimes);
        timeoffset = newtimes(1)-oldtimes(1);
        newdata = interp1([0 oldtimes],[data(1) data],newtimes);
    end
    obj.starttime=obj.starttime+timeoffset;
    obj.samprate=newsamprate;
    % Change component first component letter if necessary
    obj.metadata.component=ts_data_channel.changeComp(obj.metadata.component,newsamprate);
    
    obj.data=newdata(:)';
end

function [y, Fout]=SincResample(xin, npoints, Fin, shift, mode)
    % SincResample resamples signals to arbitrary lengths/frequencies and can
    % timeshift them.
    %
    % SincResample can be faster than INTERP or RESAMPLE when X contains many
    % relatively short data columns.
    %
    %
    % Example:
    % [Y, FOUT]=SincResample(X, NPOINTS, FIN, SHIFT, MODE)
    %
    % X is a vector or M x N  matrix where each column represents a signal.
    % NPOINTS is the number of points required in each column after
    % resampling.
    % FIN (if supplied) is the sampling rate in X (default 1 samples/sec).
    % SHIFT (if supplied) is the time (in seconds) to shift the points at which
    % interpolation will be done (default 0). SHIFT should be between ?(1/Fin).
    % SHIFT can be used to synchronize signals separated by up to ?(1/Fin)
    % in time.
    % MODE (if supplied) is a string:
    %           'sinc' to use a hamming-windowed sinc function (default)
    %           'lanczos' for a 3-lobed Lanczos function
    %
    % Y is the resulting M x NPOINTS matrix formed by resampling. The time
    % of the first sample in each column of Y is shifted by SHIFT seconds
    % relative to those in X.
    % FOUT is the effective sample rate for output Y (normalised to FIN=1 if
    % FIN is not supplied on input).
    %
    % If X is a bandlimited (filtered) signal containing components at
    % DC - 0.5Fs where Fs is the sample rate, SincResample will return
    % the signal that would have been seen with a higher sampling rate
    % (and the same filter settings). The algorithm depends on the fact that
    % the bandlimited signal is completely defined when sampling at Fs.
    %
    % SincResample returns the data convolved with a set of  time-shifted
    % windowed sinc functions, one for each of the samples [1..size(x,1)]
    % in the input signal. End-effects are reduced by adding a reflected
    % and mirrored copy of the data at the beginning and end of each column
    % before resampling.
    %
    % Note that points at the end of Y (and/or beginning if SHIFT is negative)
    % will be extrapolated beyond the boundaries of X.
    %
    % SincResample is a generalization of the method used in Blanche &
    % Swindale (2006).
    %
    % See also e.g. interp & resample (in the Signal Processing Toolbox) and
    % the sinc online documentation.
    %
    % References:
    % T.Blanche & N.V.Swindale (2006)
    % Nyquist interpolation improves neuron yield in multiunit recordings
    % Journal of Neuroscience Methods 155, 81-91 <a href="http://dx.doi.org/doi:10.1016/j.jneumeth.2005.12.031">LinkOut</a>
    %
    % also
    %
    % K. Turkowski, Filters for common resampling tasks <a href="http://www.worldserver.com/turk/computergraphics/ResamplingFilters.pdf">LinkOut</a>
    %
    % Toolboxes required: None
    %
    % Author: Malcolm Lidierth 07/06
    % Copyright ? King?s College London 2006-8
    %
    % Acknowledgements: Steve Alty for comments
    %
    % Revisions: 10.07  Argument checks tidied (ML)
    %            02.08  End-effect minimization now relects and mirrors data
    %                   instead of simply reflecting it. This improves the
    %                   slopes at the end and reduces the behaviour described at
    %                   <a href="http://www.dsprelated.com/showmessage/79676/2.php">http://www.dsprelated.com/showmessage/79676/2.php</a>
    %                   (ML)
    %            02.08  Mode option added (ML)
    %            11.11  Added global variables to avoid repeating expensive
    %                   sinc calculations if they've already been done (WCC)
    
    % Argument checks
    if nargin<2
        y=[];
        Fout=0;
        return
    end
    
    if nargin<=2
        % Default
        Fin=1;
    end
    
    if nargin<=3
        % Zero shift default
        shift=0;
    end
    
    if nargin<=4
        % Set sinc as default
        mode='sinc';
    end
    
    if Fin==0
        error('SincResample: Fin specified as zero');
    end
    
    if abs(shift)>=1/Fin
        error('SincResample: shift must be less than 1/Fin');
    end
    
    if rem(npoints,1)~=0
        error('SincResample: npoints must be a whole number');
    end
    
    % Main routine
    % Prepend and append a reflected and mirrored copy of the data
    % to minimize end transients:
    % Change 22.02.08: note change from reflected only
    
    if isvector(xin)
        % Force column vector if row input
        xin=xin(:);
    end
    
    x=zeros(size(xin,1)*3, size(xin,2));
    for k=1:size(xin,2)
        x(2:end-1,k)=[2*xin(1,k)-xin(end:-1:2,k);...
            xin(:,k);...
            2*xin(end,k)-xin(end-1:-1:1,k)];
        x(1,k)=2*x(1,k)-x(2,k);
        x(end,k)=2*x(end,k)-x(end-1,k);
    end
    clear('xin');
    np=npoints*3;
    
    % A column vector of sample times for x  - assume unit spacing and zero
    % base
    len=size(x,1);
    global SincResample_MULT SincResample_LEN SincResample_MODE SincResample_W
    SincResample_LEN=-1;    % Don't reuse old SincResample (WCC)
    if isempty(SincResample_LEN), SincResample_LEN=-1; end
    if ((SincResample_LEN ~= len) || ~strcmp(mode,SincResample_MODE))
        ts_verbose(1,'Recalculating interpolation function');
        if (SincResample_LEN ~= len)
            ts_verbose(1,'Old length=%d, new length=%d',SincResample_LEN,len);
        else
            ts_verbose(1,'Old mode=%s, new mode=%s',SincResample_MODE,mode);
        end
        % Must recalculate interp function
        t=(0:len-1)';
        
        % times at which to interpolate data - based on spacing in t.
        % ts will be a [npoints by size(x,1)] matrix
        ts=(0:(max(t)+1)/np:(np-1)/(np/len))';
        ts=(ts(:,ones(size(t)))-t(:,ones(size(ts)))');
        
        % Hamming window
        N=size(ts,2);
        if N==1 || strcmp(mode, 'lanczos');
            w=1;
        else
            th=ts+N-1;
            w=0.54-0.46*cos((2*pi*th/max(th(:)))) ;
        end
        
        % Add shift in multiples of a sampling interval i.e. shift/(1/Fin)
        ts=ts+(shift*Fin);
        switch mode
            case 'lanczos'
                % Generate Lanczos functions and calculate output
                mult=lanczos(ts);
                y=mult.*x;
            case 'sinc'
                % Generate sinc functions, apply Hamming window if w~=1 and
                % calculate output.
                mult=sinc(ts);
                y=mult.*w*x;
                SincResample_W= w;
            otherwise
                error('mode should be ''sinc'' or ''lanczos''');
        end
        SincResample_MULT= mult;
        SincResample_MODE= mode;
        SincResample_LEN = len;
    else
        switch SincResample_MODE
            case 'lanczos'
                y=SincResample_MULT.*x;
            case 'sinc'
                y=SincResample_MULT.*SincResample_W*x;
            otherwise
                error('mode should be ''sinc'' or ''lanczos''');
        end
    end
    
    % Throw away what is not needed
    y=y(size(y,1)/3+1:(size(y,1)/3)+(size(y,1)/3),:);
    Fout=Fin*npoints/(len/3);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % DEBUG Display the data - UNCOMMENT text below to display results
    % x=x(size(x,1)/3+1:(size(x,1)/3)+(size(x,1)/3),:);
    % debugdisplay(x, y, Fin, Fout, shift)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    return
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y=sinc(x)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    i=find(x==0);
    x(i)=1;
    y=sin(pi*x)./(pi*x);
    y(i)=1;
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y=lanczos(x)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    n=3;
    y=sinc(x).*sinc(x/n);
    y(abs(x)>=n)=0;
    return
end

