function txt = char(obj,fmt)
%Generate text describing the ts_datetime object
% Usage: string = char(obj,format)
%   format an optional free-form format containing characters and any of
%          the following symbols:
%           '%y' 4-digit year
%           '%m' 2-digit month
%           '%d' 2-digit day
%           '%j' 3-digit julian day
%           '%H' 2-digit hour
%           '%M' 2-digit minute
%           '%S' 2-digit seconds
%           '%F' 3-digit milliseconds
%           '%U' 6-digit microseconds
%
%   the default format is '%y/%m/%d %H:%M:%S.%F' (e.g. '2004/02/0412:00:00.000'
%
%   there are also a few predefined formats:
%           'YMD'    => '%y/%m/%d'    (e.g. '2004/02/04')
%           'YJ'        '%y/%j'       (e.g. '2004/035')
%           'HMS'       '%H:%M:%S'    (e.g. '12:00:00')
%           'compact'   '%y%m%d.%H%M' (e.g. '20040204.1200')

%  W Crawford 12/2008
if obj==ts_datetime();  % Equals default (NULL) value
    txt='NULL'; return;
end
if ~exist('fmt','var'), fmt='%y/%m/%d %H:%M:%S.%F'; end
switch upper(fmt)
    case 'COMPACT', fmt='%y%m%d.%H%M';
    case 'YMD',     fmt='%y/%m/%d';
    case 'YJ',      fmt='%y/%j';
    case 'HMS',     fmt='%H:%M:%S';
end
objarr=obj; txt=[];
for il=1:length(objarr)    % Handles arrays of objects
    obj=objarr(il);
    intsecs = floor(obj.second);
    
    ifmt=1; nfmt=length(fmt);
    str='';
    
    while ifmt<=nfmt,
        if fmt(ifmt)=='%',
            fsymb = fmt(ifmt+1);
            ifmt = ifmt+2;
            switch fsymb,
                case 'y',
                    str = sprintf('%s%04d',str,obj.year);
                case 'm',
                    str = sprintf('%s%02d',str,obj.month);
                case 'd',
                    str = sprintf('%s%02d',str,obj.day);
                case 'j',
                    str = sprintf('%s%03d',str,obj.jday);
                case 'H',
                    str = sprintf('%s%02d',str,obj.hour);
                case 'M',
                    str = sprintf('%s%02d',str,obj.minute);
                case 'S',
                    str = sprintf('%s%02d',str,intsecs);
                case 'F',
                    millisecs = round(1000*(obj.second - intsecs));
                    if millisecs==1000, millisecs=999; end
                    str = sprintf('%s%03d',str,millisecs);
                case 'U',
                    microsecs = round(1000000*(obj.second - intsecs));
                    if microsecs==1000000, microsecs=999999; end
                    str = sprintf('%s%06d',str,microsecs);
                otherwise,
                    % Just stuff unknown formats back into string
                    str = [str '%' fsymb];
            end
        else
            % Stuff non-format characters into string
            str = [str fmt(ifmt)];
            ifmt = ifmt+1;
        end
    end
    if il==1, txt=str;
    else txt=sprintf('%s\n%s',txt,str);
    end
end
