function         [y,j,H,M,S]=strptime(str,format)
% Extract date and time from a string
%
% Inputs:
%   str    a text string
%   fmt    an optional free-form format containing characters and any of
%           the following symbols:
%               '%y' year
%               '%m' month
%               '%d' day
%               '%j' julian day (do not use month or day!)
%               '%H' hour
%               '%M' minute
%               '%S' seconds
%           if the format string does not have spaces or other characters
%           between the symbols, the following lengths will be assumed:
%               4 digits: %y
%               3 digits: %j
%               2 digits: %m, %d, %H, %M
%
% Output:
%   out   a ts_datetime object
%
% The following formats are automatically interpreted:
%   '%y/%m/%d %H:%M:%S' '%y/%m/%d %H:%M' '%y/%m/%d' 
%      '%y/%j %H:%M:%S'    '%y/%j %H:%M'    '%y/%j'
%
%	By the way, if you say Y=99, the year will be 99,
%			so enter 1999, etc.

if ~exist('str','var'), error('No string specified'), end
if ~ischar(str), error('"str" was not a string!'); end
ts_debug(1,'Starting: string="%s"',str);
%class(str)
if ischar(str)
    if ~exist('format','var'),
        numslash = length(findstr(str,'/'));
        numcolons=length(findstr(str,':'));
        numdash=length(findstr(str,'-'));
        if numslash == 1,
            if numcolons==2,
                format = '%y/%j %H:%M:%S';
            elseif numcolons==1,
                format = '%y/%j %H:%M';
            else
                [temp,count]=sscanf(str,'%*d/%*d %d');
                if count==0,
                    format='%y/%j';
                else
                    format='%y/%j %H';
                end
            end
        elseif numslash == 2,
            if numcolons==2,
                format = '%y/%m/%d %H:%M:%S';
            elseif numcolons==1,
                format = '%y/%m/%d %H:%M';
            else
                [temp,count]=sscanf(str,'%*d/%*d/%*d %d');
                if count==0,
                    format='%y/%m/%d';
                else
                    format='%y/%m/%d %H';
                end
            end
       elseif numslash == 0 % Assume ISO 8601 
           if numdash==2,  
                 format = '%y-%m-%dT%H:%M:%S';
           elseif numdash==0,  
                 format = '%y%m%dT%H%M%S';
           else
               error('Can''t figure out format for datetime string: %s',str);
           end
      else   
            error('Can''t figure out format for datetime string: %s',str)
        end
        ts_debug(2,'No format provided, set to "%s" based on str',format);
    else
        ts_debug(2,'format="%s"',format);
    end
    
    % Parse according to format
    [y,j,H,M,S] = parsedtstring(str,format);
else error('Illegal type (%s) for str',class(str));
end
ts_debug(1,'Ending: YYJJJ_HHMMSS=%04d%03d_%02d%02d%02d',y,j,H,M,round(S));
return;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [y,j,H,M,S] = parsedtstring(str,fmt)
y=0;j=1;H=0;M=0;S=0;
istr=1;ifmt=1;
nfmt=length(fmt);
while ifmt <= nfmt,
    if fmt(ifmt)=='%',     % Translate a formatted character
        fmttype=fmt(ifmt+1);
        strleft=str(istr:end);
        %ts_debug(2,'%%%s:   %-20s',fmttype,strleft); 
        fixedLength=0;  % Is next number to read fixed length?
        if ifmt <= nfmt-2,
            if fmt(ifmt+2)=='%', fixedLength=1; end
        end
        ifmt=ifmt+2;
        switch fmttype,
            case 'y',
                [y,cnt] = readInt(fixedLength,strleft,4);
                ts_debug(2,'%%%s: %24s=> year=  %d',fmttype,strleft,y);
            case 'm',
                [month,cnt] = readInt(fixedLength,strleft,2);
                ts_debug(2,'%%%s: %24s=> month= %d',fmttype,strleft,month); 
           case 'd',
                [day,cnt] = readInt(fixedLength,strleft,2);
                ts_debug(2,'%%%s: %24s=> day=   %d',fmttype,strleft,day); 
           case 'j',
                [j,cnt] = readInt(fixedLength,strleft,3);
                ts_debug(2,'%%%s: %24s=> jday=  %d',fmttype,strleft,j);
            case 'H',
                [H,cnt] = readInt(fixedLength,strleft,2);
                ts_debug(2,'%%%s: %24s=> hour=  %d',fmttype,strleft,H);
            case 'M',
                [M,cnt] = readInt(fixedLength,strleft,2);
                ts_debug(2,'%%%s: %24s=> minute=%d',fmttype,strleft,M);
            case 'S',   
                if fixedLength, error('Cannot use a fixed length for seconds'); end
                [S,cnt] = readFloat(strleft);
                 ts_debug(2,'%%%s: %24s=> second=%f',fmttype,strleft,S);
           otherwise,
                error('Unknown symbol: %s',fmttype)
        end        
        istr=istr+cnt;
    else   % Read through other characters
        c=fmt(ifmt);
        %fprintf(' %s:   %s\n',c,str(istr:end));    % DEBUG
        ifmt=ifmt+1;
        if c==' ',
            while isspace(str(istr)),
                istr = istr+1;
            end
        else
            if str(istr)==c, istr = istr+1;
            else error('format character ''%s'' does not match string at column %d',c,istr);
            end
        end
    end
end
if exist('month','var') || exist ('day','var'),
    if exist('day','var')==0,
        error('Specified month without specifying day');
    elseif exist('month','var')==0,
        error('Specified day without specifying month');
    elseif exist('y','var')==0,
        error('Cannot calculate julian day from month and day without specifying year');
    end
    j=mdaytojday(y,month,day);
end
return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [val,count] = readInt(fixedLen,strleft,deflen)

    if fixedLen,
        [val,count,ERRMSG,NEXTINDEX]=sscanf(strleft,['%' num2str(deflen) 'd'],1);
    else
        [val,count,ERRMSG,NEXTINDEX]=sscanf(strleft,'%d',1);
    end
    
    count=NEXTINDEX-1;
return;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [val,count] = readFloat(strleft)

    [val,count,ERRMSG,NEXTINDEX]=sscanf(strleft,'%f',1);
    
    count=NEXTINDEX-1;
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function jday = mdaytojday(year,month,day)
% Calculates julian day from
%			the year, month and day

if month>12, error('Specified month=%d: must be <= 12 ',month); end
if month<1,  error('Specified month=%d: must be >= 1',month); end
mlen = [31 28 31 30 31 30 31 31 30 31 30 31];
if ts_datetime.isleapyear(year), mlen(2) = 29; end
jday = day;
for i = 1:month-1,
	jday = jday+mlen(i);
end
                
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
