classdef ts_datetime
    % Class for dates and times
    %
    % EXAMPLE:
    % dt1=ts_datetime(2007,04,05,12,00,00)
    % dt2=ts_datetime(2007,95,12,00,00);
    % dt3=ts_datetime('2007/04/05 12:00')
    % dt4=ts_datetime('2007/95 12:00')
    % dt1==dt2
    % dt2==dt3
    % dt3==dt4
    % dt4-dt3
    % dt4=dt4+86400
    % dt4-dt3
    % max(dt3,dt4)
    % ts_datetime.str2seconds('3h')
    %
    
    % PUBLIC PROPERTIES
    properties
        year = 0;  % Year
        jday = 1;  % Integer Julian day
        hour = 0;  % Hour
        minute=0;  % Minute
        second=0;  % Second
    end
    
    % DEPENDENT PROPERTIES
    properties (Dependent = true)
        month=0;    % Month of year (Dependent on year and jday)
        day=0;      % Dependent property (Dependent on year and jday)
    end % properties
    
    %% PUBLIC METHODS
    methods
        %% CREATOR
        function DT =  ts_datetime(varargin)
            % Input time as a string, a list of numbers, or a Matlab serial date
            % time. Valid inputs are:
            %   ts_datetime(y,m,d,H,M,S)
            %   ts_datetime(y,julianDay,H,M,S)
            %   ts_datetime(y,m,d)
            %   ts_datetime(y,julianDay)
            %   ts_datetime('yyyy/m/d H:M:S')
            %   ts_datetime('yyyy/m/d H:M')
            %   ts_datetime('yyyy/m/d')
            %   ts_datetime('yyyy/jday H:M:S')
            %   ts_datetime('yyyy/jday H:M')
            %   ts_datetime('yyyy/jday')
            %   ts_datetime(string,fmt)
            %   ts_datetime(matlabSerialTimeNumber)
            %   GRANDFATHERED (don't use, might be removed in future)
            %   ts_datetime([y m d H M S])  (numeric vector)
            %   ts_datetime([y julianDay H M S]) (numeric vector)
            %
            % 'fmt' is a free-form format containing characters and any of the following symbols:
            %               '%y' year
            %               '%m' month
            %               '%d' day
            %               '%j' julian day (do not use month or day!)
            %               '%H' hour
            %               '%M' minute
            %               '%S' seconds
            %           if the format string does not have spaces or other characters
            %           between the symbols, the following lengths will be assumed:
            %               4 digits: %y
            %               3 digits: %j
            %               2 digits: %m, %d, %H, %M
            %   Examples: the following are equivalent:
            %       obj=ts_datetime(2005,4,14,22,13,42.5);
            %       obj=ts_datetime(2005,104,22,13,42.5);
            %       obj=ts_datetime('2005/4/14 22:13:42.5');
            %       obj=ts_datetime('2005/104 22:13:42.5');
            %       obj=ts_datetime('14-4-2005 221342.5','%d-%m-%y %H%M%S');
            ts_init()
            if isempty(varargin),
                DT=ts_datetime(0,1,0,0,0); % NULL value
                return;
            end
            if isa(varargin{1},'ts_datetime')
                DT=varargin{1};
            else
                H=0; M=0; S=0;
                if isnumeric(varargin{1})
                    % If a numeric array, strip out to standard cell array
                    if length(varargin{1})>1
                        temp=varargin{1};varargin={};
                        for i=1:length(temp);
                            varargin{i}=temp(i);
                        end
                    end
                    nD=length(varargin);
                    if nD ==5,    % Time entered as Y,JD,H,M,S
                        [y,j,H,M,S] = deal(varargin{:});
                    elseif nD == 6,   % Time entered as Y,MO,D,H,M,S
                        [y,m,d,H,M,S] = deal(varargin{:});
                        j =   ts_datetime.mdaytojday(y,m,d);
                    elseif nD == 3,   % Time in Y, MO, D
                        [y,m,d]=deal(varargin{:});
                        j =   ts_datetime.mdaytojday(y,m,d);
                    elseif nD == 2,   % Time in Y, JD
                        [y,j]=deal(varargin{:});
                    elseif nD==1, % Matlab serial date string
                        V=datevec(varargin{1});
                        c=num2cell(V);
                        [y,m,d,H,M,S] = deal(c{:});
                        j =   ts_datetime.mdaytojday(y,m,d);
                    else
                        error('You specified %d numeric values: only 1, 2, 3, 5,or 6 are acceptable',nD);
                    end
                elseif isa(varargin{1},'char'),    % Time entered as string
                    [y,j,H,M,S]=ts_datetime.strptime(varargin{:});
                else
                    error('Unknown inputs');
                end
                DT.year =   y;
                DT.jday =   j;
                DT.hour =   H;
                DT.minute = M;
                DT.second = S;
            end
            
        end % ts_datetime
        
        %% SET METHODS (ELIMINATE BAD INPUTS)
        function obj = set.year(obj,val)
            if length(val)>1, error('value must be a scalar');
            elseif ~isa(val,'numeric'), error('value must be an integer');
            elseif val~=round(val), error('value must be an integer');
            end
            obj.year=val;
        end
        function obj = set.jday(obj,val)
            if length(val)>1, error('value must be a scalar');
            elseif ~isa(val,'numeric'), error('value must be an integer');
            elseif val~=round(val), error('value must be an integer');
            elseif val<1 || val > 366,
                error('value must be in the range 1-366 (is %d)',val);
            end
            obj.jday=val;
        end
        function obj = set.hour(obj,val)
            if length(val)>1, error('value must be a scalar');
            elseif ~isa(val,'numeric'), error('value must be an integer');
            elseif val~=round(val), error('value must be an integer');
            elseif val<0 || val>23
                error('value must be in the range 0 to 23 (is %d)',val);
            end
            obj.hour=val;
        end
        function obj = set.minute(obj,val)
            if length(val)>1, error('value must be a scalar');
            elseif ~isa(val,'numeric'), error('value must be an integer');
            elseif val~=round(val), error('value must be an integer');
            elseif val<0 || val>59
                error('value must be in the range 0 to 59 (is %d)',val);
            end
            obj.minute=val;
        end
        function obj = set.second(obj,val)
            if length(val)>1, error('value must be a scalar');
            elseif ~isa(val,'numeric'), error('value must be an integer');
            elseif val<0. || val>60.
                error('value must be in the range 0. to 60. (is %g)',val);
            end
            obj.second=val;
        end
        %% ACCESS METHODS FOR DEPENDENT VARIABLES
        function mon = get.month(obj)
            mon = calcmday(obj);
        end
        function day = get.day(obj)
            [~,day] = calcmday(obj);
        end
        
        %% DISPLAY AND CHAR METHODS
        txt = char(obj,fmt);       % DEFINED IN SEPARATE FILE
        function disp(obj)
            % Called when a ts_datetime object is not followed by a semicolon
            disp(char(obj));
        end
        
        %% OTHER METHODS
        function obj = modify(obj,amount,fmt)
            % More flexible, string-based add/subtract time from a datetime
            % Note: This does not change the object, you must set another equal
            %   to it!
            %
            % Input:
            %   amount = (float) time to add in seconds
            %            (string) time to add in seconds (end in 's' or
            %                   nothing), minutes (end in 'm'); hours (end in
            %                   'm') or days (end in 'd').
            %                   You can also specify a complete datetime string
            %                   (see ts_datetime.strptime())) and format to
            %                   completely change the datetime
            if isa(amount,'numeric')
                obj = obj + amount;
                return;
            elseif isa(amount,'char')
                if exist('fmt','var')
                    obj = ts_datetime(amount,fmt);
                elseif ~isempty(strfind(amount,'/')),
                    obj = ts_datetime(amount);
                else
                    obj = obj + ts_datetime.str2seconds(amount);
                end
            end
        end % modify
        function sdn = toMatlabSerial(obj)
            % Returns equivalent Matlab Serial Date Number
            sdn = obj.toDateNum();
            warning('toMatlabSerial() is grandfathered, use toDateNum() instead');
        end
        function sdn = toDateNum(obj)
            % Returns equivalent Matlab Serial Date Number
            sdn = datenum(obj.year,obj.month,obj.day,obj.hour,obj.minute,obj.second);
        end
        function obj = round(obj,gran,origin)
            % Rounds off to the nearest second or other time unit
            % Usage:  mytime=round(mytime,granularity,origin)
            %   granularity can be a number of seconds, or a
            %   string:
            %       'second','s', [default]; 'minute','m,
            %       'hour', 'h'; 'day', 'd'
            %       'month'; 'year', 'y'
            %  origin [default=0] gives the origin (in seconds) to round down to
            % Examples:
            %   for dt=ts_datetime('2007/07/12 13:24:56.85');
            %     dt.round()                   % Gives 2007/07/12 13:24:57.000
            %     dt.round('hour');            % Gives 2007/07/12 14:00:00.000
            %     dt.round(600);               % Gives 2007/07/12 13:20:00.000
            %     dt.round('hour', 1800);      % Gives 2007/07/12 13:30:00.000
            %     dt.round('month');           % Gives 2007/07/01 00:00:00.000
            
            if ~exist('gran','var'), gran=1; end
            if ~exist('origin','var'), origin=0; end
            [before,after]=getGranTimes(obj,gran,origin);
            bdiff=obj-before;
            adiff=after-obj;
            if bdiff<adiff, % Before time is closer
                obj=before;
            else           % After time is closer or equal
                obj=after;
            end
        end;
        function obj = ceil(obj,gran, origin)
            % Rounds up to the next second, minute, hour, or day
            % Usage: mytime=ceil(mytime,granularity, origin)
            %   granularity can be a number of seconds, or a
            %   string:
            %       'second','s', [default]; 'minute','m,
            %       'hour', 'h'; 'day', 'd'
            %       'month'; 'year', 'y'
            %  origin [default=0] gives the origin (in seconds) to round down to
            % Examples:
            %   for dt=ts_datetime('2007/07/12 13:24:56.85');
            %     dt.ceil()                   % Gives 2007/07/12 13:24:57.000
            %     dt.ceil('hour');            % Gives 2007/07/12 14:00:00.000
            %     dt.ceil(600);               % Gives 2007/07/12 13:30:00.000
            %     dt.ceil('hour', 1800);      % Gives 2007/07/12 13:30:00.000
            %     dt.ceil('month');           % Gives 2007/08/01 00:00:00.000
            
            if ~exist('gran','var'), gran=1; end
            if ~exist('origin','var'), origin=0; end
            [before,after]=getGranTimes(obj,gran, origin);
            if obj==before, obj=before;
            else obj=after;
            end
        end;
        function obj = floor(obj,gran,origin)
            % Rounds down to the next second, minute, hour, day, month or year
            % Usage: mytime=floor(mytime,granularity, origin)
            %   granularity can be a number of seconds, or a
            %   string:
            %       'second','s', [default]; 'minute','m,
            %       'hour', 'h'; 'day', 'd'
            %       'month'; 'year', 'y'
            %  origin [default=0] gives the origin (in seconds) to round down to
            % Examples:
            %   for dt=ts_datetime('2007/07/12 13:24:56.85');
            %     dt.floor()                   % Gives 2007/07/12 13:24:56.000
            %     dt.floor('hour');            % Gives 2007/07/12 13:00:00.000
            %     dt.floor(600);               % Gives 2007/07/12 13:20:00.000
            %     dt.floor('hour', 1800);      % Gives 2007/07/12 12:30:00.000
            %     dt.floor('month');           % Gives 2007/07/01 00:00:00.000
            
            if ~exist('gran','var'), gran=1; end
            if ~exist('origin','var'), origin=0; end
            [before,after]=getGranTimes(obj,gran, origin);
            if obj==after, obj=after;
            else obj=before;
            end
        end;
        function [header,ticks,labels]=maketimeticks(starttime,endtime,maxticks)
            % Return ticks and labels for a time axis
            if ~isa(starttime,'ts_datetime')
                error('starttime must be a ts_datetime object');
            elseif ~isa(endtime,'ts_datetime')
                error('endtime must be a ts_datetime object');
            end
            if ~exist('maxticks','var')
                maxticks=6;
            end
            ts_debug(1,'Starting, starttime=%s, endtime=%s, maxticks=%d',char(starttime),char(endtime),maxticks);
            trange=endtime-starttime;
            mingap=trange/maxticks;
            
            % select tick locations
            divunits='seconds'; header='%y/%m/%d %H:%M';
            if mingap<.001,
                fmt='%S.%Us';
                delta=10^(ceil(log10(mingap)));
                delsecs=delta;
                if trange/delsecs>maxticks, delsecs=delsecs*2; end
                if trange/delsecs>maxticks, delsecs=delsecs*2.5; end
            elseif mingap<=.5,
                fmt='%S.%Fs';
                delta=10^(ceil(log10(mingap)));
                delsecs=delta;
                if trange/delsecs>maxticks, delsecs=delsecs*2; end
                if trange/delsecs>maxticks, delsecs=delsecs*2.5; end
            elseif mingap < 60,     % tick interval in seconds
                fmt='%Mm%S';
                delta=60;
                for d=[1 2 5 10 20 30]
                    if mingap<d, delta=d; break; end
                end
                delsecs=delta;
            elseif mingap < 3600;  % tick interval in minutes
                divunits = 'minutes'; mingap=mingap/60; fmt='%Hh%M';
                delta=60;
                for d=[1 2 5 10 20 30]
                    if mingap<d, delta=d; break; end
                end
                delsecs=delta*60;
            elseif mingap < 86400; % tick interval in hours
                divunits = 'hours'; mingap=mingap/3600;
                header='%y/%m'; fmt='%d %Hh';
                delta=24;
                for d=[1 2 3 4 6 12]
                    if mingap<d, delta=d; break; end
                end
                delsecs=delta*3600;
            elseif mingap < 31*86400; % tick interval in days
                divunits = 'days'; mingap=mingap/86400;
                header='%y'; fmt='%m/%d';
                delta=30;
                for d=[1 2 5 7 10 14 21]
                    if mingap<d, delta=d; break; end
                end
                delsecs=delta*86400;
            elseif mingap < 365*86400; % tick interval in months
                divunits = 'months'; mingap=mingap/(30*86400);
                header='Date'; fmt='%y/%m';
                delta=12;
                for d=[1 2 3 4 6]
                    if mingap<d, delta=d; break; end
                end
            else                        % tick interval in years
                divunits='years'; mingap=mingap/(365*86400);
                header=''; fmt='%y';
                delta=1000;
                for d=[1 2 4 5 10 20 50 100 200 500 1000]
                    if mingap<d, delta=d; break; end
                end
            end
            if exist('delsecs','var')
                time=ceil(starttime,delsecs);
            elseif strcmp(divunits,'months')
                time=ceil(starttime,'months');
            elseif  strcmp(divunits,'years')
                time=ceil(starttime,'years');
                time.year=delta*ceil(time.year/delta);
            else
                error('delsecs not defined, but we are not in months nor years');
            end
            header=char(time,header);
            ts_debug(1,'first tick time=%s, end time=%s, divunits=%s, delta=%d',char(time),char(endtime),divunits,delta);
            i=0;
            while time<endtime
                i=i+1;
                ticks(i)=time-starttime;    % seconds since start
                labels{i}=time.char(fmt);
                if exist('delsecs','var'),
                    time=time+delsecs;
                elseif strcmp(divunits,'months')
                    themonth=time.month+delta;
                    while themonth>12,
                        themonth=themonth-12;
                        time.year=time.year+1;
                    end
                    time.jday=ts_datetime.mdaytojday(time.year,themonth,1);
                elseif strcmp(divunits,'years');
                    time.year=time.year+delta;
                else
                    error('delsecs not defined, but we are not in months nor years');
                end
            end
            
        end
        
        
        
        %% OPERATORS: COMPARISON
        function result = eq(a,b)
            % Implements datetimeA == datetimeB
            if (~isa(b,'ts_datetime')),
                error('second object must also be ts_datetime');
            end
            if a.year~=b.year, result=0; return; end    % Avoids long recursive secondsPerYear calculations
            result = a-b == 0; % This can be very slow if years are far off
        end
        function result = ne(a,b)
            % Implements datetimeA ~= datetimeB
            result = ~(a==b);
        end
        function result = ge(a,b)
            % Implements datetimeA >= dateTimeB
            if (~isa(b,'ts_datetime')),
                error('second object must also be ts_datetime');
            end
            if a.year > b.year, result=1; return;
            elseif a.year < b.year, result=0; return
            end    % Avoids long recursive secondsPerYear calculations
            result = a-b >= 0;
        end
        function result = gt(a,b)
            % Implements datetimeA > dateTimeB
            if (~isa(b,'ts_datetime')),
                error('second object must also be ts_datetime');
            end
            if a.year > b.year, result=1; return;
            elseif a.year < b.year, result=0; return
            end    % Avoids long recursive secondsPerYear calculations
            result = a-b > 0;
        end
        function result = le(a,b)
            % Implements datetimeA <= dateTimeB
            if (~isa(b,'ts_datetime')),
                error('second object must also be ts_datetime');
            end
            if a.year < b.year, result=1; return;
            elseif a.year > b.year, result=0; return
            end    % Avoids long recursive secondsPerYear calculations
            result = a-b <= 0;
        end
        function result = lt(a,b)
            % Implements datetimeA < dateTimeB
            if (~isa(b,'ts_datetime')),
                error('second object must also be ts_datetime');
            end
            if a.year < b.year, result=1; return;
            elseif a.year > b.year, result=0; return
            end    % Avoids long recursive secondsPerYear calculations
            result = a-b <0;
        end
        %         function result = neq(a,b)
        %         % Implements datetimeA ~= dateTimeB
        %             if (~isa(b,'ts_datetime')),
        %                 error('second object must also be ts_datetime');
        %             end
        %             if a.year~=b.year, result=1; return; end    % Avoids long recursive secondsPerYear calculations
        %             result = a-b ~= 0;
        %         end
        
        %% OPERATORS: ADDITION/SUBTRACTION (these have been vectorized)
        function outp= minus(obj,subtractor)
            % Implements datetime - seconds -OR- datetimeA - datetimeB
            if isa(subtractor,'numeric'),
                outp = plus(obj,-subtractor);
            elseif isa(subtractor,'ts_datetime'),
                if length(obj)>1
                    outp=obj;
                    if length(subtractor)==1   % Subtract ts_datetime from vect of ts_datetimes
                        for i=1:length(obj),
                            outp(i)=obj(i)-subtractor;      % recursive call
                        end
                    elseif length(subtractor)==length(obj) % Subtract ts_datetime vector from ts_datetime vector
                        for i=1:length(obj),
                            outp(i)=obj(i)-subtractor(i);   % recursive call
                        end
                    else
                        error('object and subtractor have different lengths')
                    end
                elseif length(subtractor)>1,   % Add vector to scalar ts_datetime
                    for i=1:length(subtractor)
                        outp(i)=obj-subtractor(i);
                    end
                else    % Scalar addition
                    timeb = subtractor; timea = obj;
                    if timea.year<timeb.year,	% Swap times so ya always has >= year to yb
                        timec=timea; timea=timeb; timeb=timec;
                        mult = -1;
                    else mult=1;
                    end
                    
                    bsecs = 86400*timeb.jday + 3600*timeb.hour + 60*timeb.minute + timeb.second;
                    asecs = 86400*timea.jday + 3600*timea.hour + 60*timea.minute + timea.second;
                    %timea.year,timeb.year,class(timea),class(timeb)
                    for y = timeb.year:timea.year-1,
                        asecs = asecs + 86400*(365 + ts_datetime.isleapyear(y));
                    end
                    outp = (asecs-bsecs)*mult;
                end
            else
                error('You can only subtract a number or another ts_datetime object from a ts_datetime object');
            end
        end % function
        function obj = plus(obj,addSecs)
            % Implements datetime + seconds
            if ~isa(obj,'ts_datetime'); % if ts_datetime object is second one, flip them around
                tmp=obj;
                obj=addSecs;
                addSecs=tmp;
            end
            
            if ~isnumeric(addSecs),
                error('only a numeric value (in seconds) can be added to an ts_datetime object');
            end
            if length(obj)>1
                if length(addSecs)==1   % Add scalar to vector of ts_datetimes
                    for i=1:length(obj),
                        obj(i)=obj(i)+addSecs;      % recursive call
                    end
                elseif length(addSecs)==length(obj) % Add vector to vector of ts_datetimes
                    for i=1:length(obj),
                        obj(i)=obj(i)+addSecs(i);   % recursive call
                    end
                else
                    error('object and addSecs have different lengths')
                end
            elseif length(addSecs)>1,   % Add vector to scalar ts_datetime
                tmp=obj;
                for i=1:length(addSecs)
                    obj(i)=tmp+addSecs(i);
                end
            else    % Scalar addition
                startsecs = 86400*(obj.jday-1) + 3600*obj.hour ...
                    + 60*obj.minute + obj.second;
                for i=1:length(addSecs),
                    s = startsecs + addSecs;
                    y = obj.year;
                    ylen = 86400*(365 + ts_datetime.isleapyear(y));
                    if s < 0,	% must go back years
                        while s<0,
                            y = y-1;
                            ylen = 86400*(365 + ts_datetime.isleapyear(y));
                            s = s+ylen;
                        end
                    elseif s >=ylen,	% must go forward years
                        while s >= ylen,
                            s = s - ylen;
                            y = y+1;
                            ylen = 86400*(365 + ts_datetime.isleapyear(y));
                        end
                    end
                    obj.year = y;
                    obj.jday = floor(s/86400)+1;	s = rem(s,86400);
                    obj.hour = floor(s/3600);		s = rem(s,3600);
                    obj.minute = floor(s/60);		s = rem(s,60);
                    obj.second = s;
                end
            end
        end % function
        
        %% MAX,MIN
        function result = max(a,b)
            % Returns latest ts_datetime object
            % Usage: max(a,b)      where a and b are ts_datetime objects
            %        max(dtv)      where dtv is a vector of ts_datetime objects
            if ~exist('b','var')    % compare vector of ts_datetime objects
                result=a(1);
                for i=2:length(a);
                    if a(i)>result;
                        result=a(i);
                    end
                end
                return
            end
            if (~isa(b,'ts_datetime')),
                error('second object must also be ts_datetime');
            end
            if a>b, result=a; else result=b; end
        end
        function result = min(a,b)
            % Returns earliest ts_datetime object
            % Usage: min(a,b)      where a and b are ts_datetime objects
            %        min(dtv)      where dtv is a vector of ts_datetime objects
            if ~exist('b','var')    % compare vector of ts_datetime objects
                result=a(1);
                if length(a)==1, return; end
                for i=2:length(a);
                    if a(i)<result;
                        result=a(i);
                    end
                end
                return
            end
            if (~isa(b,'ts_datetime')),
                error('second object must also be ts_datetime');
            end
            if a<b, result=a; else result=b; end
        end
    end % public methods
    
    %% PUBLIC STATIC METHODS ( do not require a class object, call using ts_datetime.function(..) )
    methods (Static)
        function [seconds,suffix] = str2seconds(str)
            % Convert a string to a number of seconds. Old name=secondstring()
            % Usage: [seconds,suffix]=ts_datetime.str2seconds(str)
            %
            % String can end in 's' (seconds), 'm' (minutes), 'h' (hours)
            %      'd' (days), or nothing at all (seconds)
            % if "string" is a number, just returns that number
            %
            % suffix is the character at the end of the string (if any)
            if ~exist('str','var'); seconds=0; return; end
            if isa(str,'numeric'), seconds=str; return; end
            switch lower(str(end))
                case 's', seconds = str2double(str(1:end-1)); suffix='s';
                case 'm', seconds = str2double(str(1:end-1))*60; suffix='m';
                case 'h', seconds = str2double(str(1:end-1))*3600; suffix='h';
                case 'd', seconds = str2double(str(1:end-1))*86400; suffix='d';
                otherwise, seconds=str2double(str); suffix='';
            end
        end
        function str = seconds2str(seconds)
            % Convert seconds to a string
            days=floor(seconds/86400); seconds=seconds-days*86400;
            hours=floor(seconds/3600); seconds=seconds-hours*3600;
            minutes=floor(seconds/60); seconds=seconds-minutes*60;
            if days >0,
                str=sprintf('%dd, %dh, %dm',days,hours,minutes);
            elseif hours>0,
                str=sprintf('%dh, %dm, %.0fs',hours,minutes,seconds);
            elseif minutes>0,
                str=sprintf('%dm, %.0fs',minutes,seconds);
            elseif seconds>1,
                str=sprintf('%.1fs',seconds);
            elseif seconds==0,
                str='0 seconds';
            elseif seconds>.001,
                str=sprintf('%.1fms',1000*seconds);
            elseif seconds>.0000001,
                str=sprintf('%.1fus',1000000*seconds);
            else
                str=sprintf('%gs',seconds);
            end
        end
        function bool = hastimesymbols(str)
            % Determines if a string has time symbols by seeing if evaluating
            % for 2 completely different times gives different results
            bool = ~strcmp( char(ts_datetime('2000/01/01 00:00:00.000'),str) , ...
                char(ts_datetime('1940/12/12 13:13:13.998'),str) );
        end
    end % methods
    
    %% PRIVATE METHODS
    methods (Access = private)
        function [month,day] = calcmday(obj)
            % Calculate the month and day-of-month for the object
            mlen = [31 28 31 30 31 30 31 31 30 31 30 31];
            y = obj.year;
            if ts_datetime.isleapyear(y), mlen(2) = 29; end
            month = 1; day = obj.jday;
            while day > mlen(month),
                day = day - mlen(month);
                month = month+1;
            end
        end
    end
    
    %% PRIVATE STATIC METHODS ( do not require a class object, call using ts_datetime.function() )
    methods (Static, Access = private)
        [y,j,H,M,S] = strptime(string,format);             % DEFINED IN SEPARATE FILE
        function jday = mdaytojday(year,month,day)
            % Calculates julian day from the year, month and day
            mlen = [31 28 31 30 31 30 31 31 30 31 30 31];
            if ts_datetime.isleapyear(year), mlen(2) = 29; end
            jday = day;
            for i = 1:month-1,
                jday = jday+mlen(i);
            end
        end
        function resp = isleapyear(year)
            % is the input year a leap year? 1 means yes, 0 means no
            resp = (rem(year,4)==0 && (rem(year,100)~=0 || rem(year,400)==0));
        end
    end
    
    
end % classdef

function [before,after]=getGranTimes(obj,gran,origin)
    % Returns granular times directly before and after the obj time
    % origin is an offset origin time (for example, gran=3600, origin=1800,
    %       gives the half-hour closest to the input time)
    % Used by ceil, round and floor
    
    % 'month' and 'year' are treated differently from the rest because
    % there is no fixed granularity in seconds
    before = obj;
    if isnumeric(gran)
        gransecs=gran;
        % Start counting from beginning of day
        before.second=0;
        before.minute=0;
        before.hour=0;
        if gransecs > 86400
            before.jday=1;
        end
        if gransecs > 365*86400
            before.year=before.year-1;
        end
    else
        switch gran,
            case {'seconds','second','s'}
                gransecs=1;
                before.second=0;
            case {'minutes','minute','m'}
                gransecs=60;
                before.second=0;
                before.minute=0;
            case {'hours','hour','h'}
                gransecs=3600;
                before.second=0;
                before.minute=0;
                before.hour=0;
            case {'days','day','d'}
                gransecs=86400;
                before.second=0;
                before.minute=0;
                before.hour=0;
                before.jday=1;
            case {'months','month'}
                if origin ~=0,
                    ts_warning('origin offset makes no sense for granularity=''%s'': ignoring',gran);
                end
                before.second=0;
                before.minute=0;
                before.hour=0;
                before.jday=ts_datetime.mdaytojday(before.year,before.month,1);
                after=before;
                themonth=before.month+1;
                if themonth>12, themonth=themonth-12;  after.year=after.year+1; end
                after.jday=ts_datetime.mdaytojday(after.year,themonth,1);
                return; % Avoid calculation at bottom
            case {'years','year','y'}
                if origin ~=0,
                    ts_warning('origin offset makes no sense for granularity=''%s'': ignoring',gran);
                end
                before.second=0;
                before.minute=0;
                before.hour=0;
                before.jday=1;
                after=before;
                after.year=before.year+1;
                return; % Avoid calculation at bottom
            otherwise,
                fprintf('ts_datetime:round(): "%s" invalid, rounding off to seconds\n', what);
        end
    end
    
    % FOR ALL BUT MONTH AND YEAR GRANULARITIES, CALCULATE BEFORE AND AFTER
    % TIMES USING THE SAME ALGORITHM BASED ON THE # OF SECONDS IN THE
    % GRANULARITY
    before = before+origin;
    while before>obj,  % in case adding the origin made "before" after the object
        before=before-gransecs;
    end
    difference=obj-before;
    before=before+gransecs*floor(difference/gransecs);
    after=before+gransecs;
end


%         function result = colon(a,d,b)
%         % Implements datetimeA:spacing:dateTimeB
%             if (~isa(b,'ts_datetime')),
%                 error('final object must also be ts_datetime');
%             end
%             i=1;
%             while a <= b
%                 result(i)=a;
%                 i=i+1;
%                 a=c+d;
%             end
%         end
%

