classdef ts_figureproperties
    % Class for figure properties
    % used by most TiSKit classes that have a "plot" method
    %
    % ts_figureproperties properties:
    %   title        - Figure title                                     (string)
    %   bottomoffset - Offset of bottom plot from figure bottom         (double)
    %                 (in points) [0]
    %   overlay      - If/how to overlay plots:                         (double)
    %                      0=no overlay
    %                     -1=overlay with common scale
    %                     -2=overlay, each autoscaled (time plots only)
    %                   dely=overlay, each autoscaled and offset by
    %                        dely of full scale (time plots only)
    %   yscale       - y-axis scale, valid values are:                  (string)
    %                     'default', 'linear', 'log', 'db'
    %   fontsize     - Font size [10]                                   (integer)
    %   xticks       - Specify x ticks, empty uses default              (vector)
    %   xticklabels  - Specify x tick labels, empty uses default        (cell vector)
    %   xlabel       - x label, empty uses default                      (string)
    %   plotasmatrix - Organize plots in a matrix (COHERENCE ONLY?)     (boolean)
    %                  [0]
    %   nSubplots    - Number of subplots                               (integer)
    %   subplot      - Vector of subplot parameters                     (ts_figureproperties_subplot)    
    %
    % ts_figureproperties methods:
    %   ts_figureproperties - Creator
    %   addtitle            - Add a title to the plot (called by plot routines)
    %   calcplotpos         - Calculate the position for a subplot
    
    properties
        title='';           % Figure title
        bottomoffset=0;     % Offset from bottom of figure (points)
        overlay=0;          % 0=no overlay, -1=overlay with common scale,-2=overlay, each autoscaled (time plots only)
                            % dely=plot together, each autoscaled and offset by dely of full scale (time plots only)
        yscale='default'    % y-axis scale: 'default', 'linear', 'log', 'db'
        fontsize=10;        % Font size
        xticks=[];          % Specify x ticks, empty uses default
        xticklabels={};     % Specify x tick labels, empty uses default
        xlabel=[];          % x label, empty uses default
        plotasmatrix = 0;	% COHERENCE ONLY? plots as matrix [0]
        nSubplots = 1;    % Number of subplots
        subplot= ts_figureproperties_subplot();   % Subplot properties      (ts_figureproperties_subplot')
    end
    
    %% METHODS
    methods
        % CREATOR METHOD
        function obj=ts_figureproperties(varargin)
            % Create a ts_figureproperties object
            % Usage:
            %   obj = ts_figureproperties(nSubplots,setargs...)
            %
            % The subplot-specific parameter array should either have as many
            % elements as there are channels, or just one value (shared among channels)
            
            % DO NOT CALL ts_init or ts_set, since ts_init calls ts_figureproperties!
            
            % Choose between argument lists
            
            % If # of subplots is specified, make new subplot instance for each one
            numargs = length(varargin);
            if rem(numargs,2) == 1,	% Odd # of arguments: First argument is the # of channels
                obj.nSubplots = varargin{1};
                % Make as many subplot fields as there are subplots
                for i = 2:obj.nSubplots,
                    obj.subplot(i) = obj.subplot(1);
                end
                if numargs >1, varargin = varargin{2:numargs}; else return; end
            end
            
            % IF NO PROPERTIES TO SET, return empty version
            if isempty(varargin),
                return;
            else    % OTHERWISE SET PROPERTIES
                for i=1:2:length(varargin)
                    obj.set(varargin{i},varargin{i+1});
                end
            end
        end
        
        %% PROPERTY ACCESS METHODS: SET
        function obj=set.title(obj,value)
            if isa(value,'char');
                obj.title=value;
            else
                error('value must be char');
            end
        end
        function obj=set.xticks(obj,value)
            if isnumeric(value);
                obj.xticks=value;
            else
                error('value must be numeric');
            end
        end
        function obj=set.xticklabels(obj,value)
            if isa(value,'cell');
                obj.xticklabels=value;
            else
                error('value must be cell array');
            end
        end
        function obj=set.xlabel(obj,value)
            if ischar(value);
                obj.xlabel=value;
            else
                error('value must be char');
            end
        end
        function obj=set.bottomoffset(obj,value)
            if isa(value,'numeric') && length(value)==1;
                obj.bottomoffset=value;
            else
                error('value must be char');
            end
        end
        function obj = set.overlay(obj,value)
            if ~isnumeric(value)
                error('value must be numeric');
            end
            obj.overlay=value;
        end    
        function obj=set.fontsize(obj,value)    % Handles non-integers
            obj.fontsize=round(value);
        end
        function obj=set.plotasmatrix(obj,value)    % changed to 1 or 0 instead of text
            if ischar(value)
                switch lower(value)
                    case 'on', obj.plotasmatrix=1;
                    case 'off', obj.plotasmatrix=0;
                    otherwise, error('Illegal plotasmatrix argument: %s',value);
                end
            else
                obj.plotasmatrix = obj.plotasmatrix~=0;
            end
        end       
        function obj=set.nSubplots(obj,value)
            oldsubplots = obj.nSubplots;
            obj.nSubplots = round(value);
            if oldsubplots > obj.nSubplots,	% Remove channels
                obj.subplot = obj.subplot(1:obj.nSubplots);
            elseif oldsubplots < obj.nSubplots,	% Add channels
                ts_verbose(1,'Adding %d channels',obj.nSubplots-oldsubplots);
                temp=obj.subplot(1);
                %obj.subplot=temp;
                obj.subplot(oldsubplots+1:obj.nSubplots) = obj.subplot(1);
            end
        end        
        function obj=set.yscale(obj,value)
            if ~isa(value,'char');
                error('value must be char');
            end
            switch lower(value),
                case {'default','linear','log','db'},
                    obj.yscale = lower(value);
                otherwise, ts_warning('yscale unchanged: should be "default","linear","log" or "db"');
            end
        end     
          function obj=set.subplot(obj,value)
             if isa(value,'ts_figureproperties_subplot');
%                  for i=1:obj.nSubplots,
%                      obj.subplot(i) = value;
%                  end
                     obj.subplot = value;
             else
                 ts_warning('value was not a subplot: nothing changed');
             end
         end
        
        %% DISPLAY AND CHAR METHODS
        function otxt=char(obj,padding)
            % Make character string describing the object
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            
            otxt=      sprintf('%s%20s: ''%s''\n',  pad,'title',obj.title);
            otxt=[otxt sprintf('%s%20s: %d\n',  pad,'bottomoffset',obj.bottomoffset)];
            otxt=[otxt sprintf('%s%20s: %d\n',  pad,'overlay',obj.overlay)];
            otxt=[otxt sprintf('%s%20s: %d elements\n',  pad,'xticks',length(obj.xticks))];
            otxt=[otxt sprintf('%s%20s: %d elements\n',  pad,'xticklabels',length(obj.xticklabels))];
            otxt=[otxt sprintf('%s%20s: ''%s''\n',  pad,'xlabel',obj.xlabel)];
            otxt=[otxt sprintf('%s%20s: ''%s''\n',  pad,'yscale',obj.yscale)];
            otxt=[otxt sprintf('%s%20s: %d\n',  pad,'fontsize',obj.fontsize)];
            otxt=[otxt sprintf('%s%20s: %d\n',    pad,'plotasmatrix',obj.plotasmatrix)];
            otxt=[otxt sprintf('%s%20s: %d\n',    pad,'nSubplots',obj.nSubplots)];
            otxt=[otxt sprintf('%s%20s: %s object\n',pad,'subplot',class(obj.subplot))];
            otxt=[otxt sprintf('%s%s',          pad,char(obj.subplot,20))];
        end
        function disp(obj)
            disp(char(obj));
        end
        
        %% OTHER METHODS
        [boundsbox,info] = calcplotpos(varargin);  % DEFINED IN SEPARATE FILE
        
        function addtitle(obj,theTitle)
            % Add title to a plot
            % uses text in obj.title, if prefix is specified, adds before
            ax = axes( 'Units', 'Normal', 'Position', [0.01 0.98 0.98 0.01], 'Visible', 'off' ) ;
            set( get( ax, 'Title' ), 'Visible', 'on' ) ;
            if ~exist('theTitle','var'), theTitle=[]; end
            if ~isempty(theTitle)
                if ~isempty(obj.title), 
                    theTitle=[theTitle ' : ' obj.title];
                end
            elseif ~isempty(obj.title)
                theTitle=obj.title;
            end
            if ~isempty(theTitle)
                text(.5,1,theTitle,'Units','Normalized', ...
                    'HorizontalAlignment','center','VerticalAlignment','top', ...
                    'Interpreter','none');
            end
        end        
    end % methods
end % classdef