classdef ts_figureproperties_subplot
    % Class for individual plot parameters
    %
    % ts_figureproperties_subplot properties:
    %     grid        - Plot Grid: 'x','y','xy','off'       ['xy']
    %     xmin        - Minimum x-axis value                [NaN]
    %     xmax        - Maximum x-axis value                [NaN]
    %     ymin        - Minimum y-axis value                [NaN]
    %     ymax        - Maximum y-axis value                [NaN]
    %     xyminmax    - READONLY, returns [xmin xmax ymin ymax]
    %     linetype    - Linetype                            ['-']
    %     marker      - Symbol type                         ['none']
    %     yrange      - Y range                             [NaN]
    %     removegain  - Remove instrument gain?             ['off']
    %     demultiplex - Demultiplex  signal into            [0]
    %                               X multiplexed channels
    %                               (0=do not demultiplex)
    %
    % ts_figureproperties_subplot methods:
    %     subsasgn   - Allows assigning a value to multiple
    %                  instances using parentheses notation
    
    
    properties
    	grid='xy';          % Grid type: 'x','y','xy','off'
		xmin=NaN            % Min x-axis value
		xmax=NaN            % Max x-axis value
		ymin=NaN;           % Min y-axis value
		ymax=NaN;           % Max y-axis value
		linetype='-'        % Linetype
		marker='none';      % Symbol type
		yrange=NaN          % TIME: Y range
		removegain='off';   % TIME: Remove instrument gain
		demultiplex=0;      % TIME: Demultiplex # of multiplexed channels (0=none)
    end
    properties (Dependent=true)
        xyminmax            % [xmin xmax ymin ymax]
    end
    
    methods
        %%CREATOR
        %function obj=ts_figureproperties_subplot()
        %end
        
        %% PROPERTY ACCESS METHODS
        function obj=set.grid(obj,value)
            switch lower(value),
                case {'x','y','xy','off'},
                    obj.grid=lower(value);
                otherwise
                    ts_warning('Invalid grid value: %s',value);
            end
        end
        
        function out=get.xyminmax(obj)
            out=[obj.xmin obj.xmax obj.ymin obj.ymax];
        end
        function obj=set.removegain(obj,value)
            if ischar(value),
                switch lower(value),
                    case 'off',
                        obj.removegain=0;
                    case 'on',
                        obj.removegain=1;
                    otherwise
                        ts_warning('Invalid t_removegain value: %s', value);
                end
            else obj.removegain=value~=0;
            end
        end
        
        function obj=subsasgn(obj,s,val)    
            % Allows multiple object assignment using (:) and (m:n) notations
            %
            % EXAMPLE:
            %  How does this work??????
            
            if isempty(s) && isa(val,'ts_supplotparm')   % No subscripts, direct assign
                obj=val;
            end
            switch s(1).type  
                case '()'  % if '(:)' or '(m:n)' and single input, assign single to all
                    if length(obj)>1 && numel(val)==1 && length(s(1).subs)==1,
                        subs=s(1).subs{1};
                        if subs==':',
                            subs=1:length(obj);
                        end
                        for i=subs,
                            s(1).subs{1}=i;
                            obj=builtin('subsasgn',obj,s,val);
                        end
                    else
                        obj=builtin('subsasgn',obj,s,val);
                    end
                    obj=builtin('subsasgn',obj,s,val);
                case '.'	% Use built-in subsasgn for dot notation
                         obj=builtin('subsasgn',obj,s,val);
                case '{}'
                    error('ts_supplotparm:subsasgn','Not a supported subscripted assignment');
            end
        end
                            
        function otxt = char(obj,padding)
            % Make character string describing the object
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');            
            iL=1:length(obj);
            otxt =       sprintf('%s%15s  %s\n', pad,'',sprintf('%-8d ',iL));
            tmp=[]; for i=iL, tmp=[tmp sprintf('%-8s ',obj(i).grid)]; end
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'grid',tmp)];
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'xmin',sprintf('%-8d ',obj(:).xmin))];
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'xmax',sprintf('%-8d ',obj(:).xmax))];
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'ymin',sprintf('%-8d ',obj(:).ymin))];
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'ymax',sprintf('%-8d ',obj(:).ymax))];
            tmp=[]; for i=iL, tmp=[tmp sprintf('%-8s ',obj(i).linetype)]; end
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'linetype',tmp)];
            tmp=[]; for i=iL, tmp=[tmp sprintf('%-8s ',obj(i).marker)]; end
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'marker',tmp)];
            otxt = [otxt sprintf('\n%s%15s: %s\n', pad,'yrange',sprintf('%-8g ',obj(:).yrange))];
            tmp=[]; for i=iL, tmp=[tmp sprintf('%-8s ',obj(i).removegain)]; end
            otxt = [otxt sprintf('%s%15s: %s\n', pad,'removegain',tmp)];
            otxt = [otxt sprintf('%s%15s: %s',   pad,'demultiplex',sprintf('%-8d ',obj(:).demultiplex))];
       end
        
        function disp(obj)
            disp(char(obj));
        end        
        
    end % methods
end % classdef
