function [tdata,dfile] = readMSEED(dfile,~,readLen)
    % READMSEED Reads miniSEED data into a ts_data object
    % tdata = readMSEED(theFile) reads the entire file into an ts_data object
    % tdata = readMSEED(theFile,startAt,readLen) reads part of the file
    %
    %	theFile can be a filename or an ts_file object with the filename
    %		parameter set to the correct value.  If the filename has wildcards,
    %		ts_file.read() will try to read all of the relevant files.
    %	startAt is ignored
    %	readLen is only used if set to -1, in which case the routine returns
    %       after reading the file's metadata
    %
    % Uses Francois Beauducel's "rdmseeed.m" function to efficiently read miniSEED files
    % http://www.ipgp.fr/~beaudu
    
    %%%%%%%% START ALL read* WITH THESE LINES
    ts_debug(1,'Starting\n');
    tic;
    %if ~exist('startAt','var'), startAt=0; end
    if ~exist('readLen','var'), readLen=0; end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if dfile.ch_metadata.tzoffset,
        error('Time-zero correction should be taken care of in ts_file.read(), not here');
    end
    
    % GET FILE INFORMATION
    dfile=getFileInfo(dfile);

    % IF READLEN < 0, return the file information without reading data
    if isa(readLen,'numeric'); if readLen<0, tdata=ts_data([],1,dfile); return; end; end
    
    % OTHERWISE, READ THE DATA
    tdata=readTheData(dfile);  
    ts_debug(1,'Ending after %s seconds\n',toc);
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function tdata=readTheData(dfile)
    % Function to read the data file.  All of the file information you need to read
    % should be in the ts_file object dfile.
    % If there are special parameters that need to be specified for your
    % data, they should be stored as a cell array in dfile.custom,
    % and their format described in the comments at the top of this file.
    
    fname =	dfile.filename;
    ts_verbose(1,'Reading miniSEED file %s',fname);
    
    [X, Info] = ts_rdmseed(dfile.filename); % F Beauducel program, plus some ts_debug stuff
    tdata = ts_data([],0,dfile);
    for i=1:length(Info)    % For each separately identified channel
        k= Info(i).XBlockIndex;
        [chdata,chtime,samprate]=verifyData(X(k));
        startTime=ts_datetime(chtime(1));  % Assumes ts_datetime reads datenum format
        %fprintf('starttime=%s\n',char(startTime)) % DEBUG
            a=regexp(Info(i).ChannelFullName,':', 'split');
        for ia = 1:length(a), a{ia}=strtrim(a{ia});end
        [network,station,location,component]=deal(a{:});
        
        % Stuff the ts_data_channel object
        tdata.channel(i).samprate=samprate;
        tdata.channel(i).starttime=startTime;
        tdata.channel(i).data=chdata;
        tdata.channel(i).metadata.network=network;
        tdata.channel(i).metadata.station=station;
        tdata.channel(i).metadata.location=location;
        tdata.channel(i).metadata.component=component;
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [chdata,chtime,samprate]=verifyData(X)
    % Check for various problems: variable sample rates, data gaps...
    debug=0;
    
    chdata=cat(1,X.d);
    chtime=cat(1,X.t); % Matlab DATENUM format
    chdata=chdata(:)';   % Make sure it's a row vector
    chtime=chtime(:)';   % Make sure it's a row vector
    %diffs=diff(chtime*86400); sps=diffs.^(-1);  figure(1); plot(sps); 
    samprate=X(1).SampleRate; 
    if ~all([X.SampleRate] == samprate),
        error('Can not read into TiSKit: same channel has multiple sample rates');
    end
    chsamps=(chtime-chtime(1))*86400*samprate;  % convert to samples from first
    chsamps=chsamps(:)'; 
    %fprintf('maxdiff=%g, mindiff=%g\n',max(diff(chsamps)),min(diff(chsamps)));  % DEBUG
    
    % Check for bad offsets between consecutive samples
    condition=diff(chsamps)<0.5;
    if any(condition),   % there are overlapped data, try to fix
        ts_warning('Fixing %d overlapped blocks',length(find(condition)))
        diffsamps=diff(chsamps);
        ind=find(diffsamps<0);
        for j=fliplr(ind)
            iSubSamps=round(-diffsamps(j));
            chsamps=[chsamps(1:j) chsamps((j+1+iSubSamps):end)];
            chdata=[chdata(1:j) chdata((j+1+iSubSamps):end)];
        end
    end
    condition=diff(chsamps)>1.5;
    if any(condition)				% there are data gaps, interpolate
        ts_warning('Filling %d data gaps',length(find(condition)));
        if (max(chsamps)-min(chsamps) > 2*length(chsamps))      % if gaps are more than 50%
            error('More than 50% gaps, quitting!')
        end
        diffsamps=diff(chsamps);
        ind=find(diffsamps>1.5);
        for j=fliplr(ind)	% Go from end to beginning, so as to not changes addresses
            iAddSamps=round(diffsamps(j))-1;
            chsamps=[chsamps(1:j-1)  chsamps(j-1)+(1:iAddSamps)  chsamps(j:end)];
            %chdata= [ chdata(1:j-1)  NaN(1,iAddSamps)  chdata(j:end)];
            chdata= [ chdata(1:j-1)  chdata(j-1)+(chdata(j)-chdata(j-1))*(1:iAddSamps)/(iAddSamps+1)  chdata(j:end)];
        end
    end
    
    % Check for bad offsets w.r.t. first sample
    N=length(chsamps);
    %whos
    offsets=chsamps-(0:(N-1)); % offset from ideal spacing
    if debug,
        figure(999); plot(offsets)   % DEBUG
        hold on
        plot([1 length(offsets)],[0.5 0.5],'--')
        plot([1 length(offsets)],-[0.5 0.5],'--')
        hold off
        fprintf('minoffset=%g, maxoffset=%g, anyabs>0.5=%d\n',min(offsets), max(offsets),any(abs(offsets) > 0.5))  % DEBUG
        input('Hit RETURN to continue')
    end
    if any(  abs(offsets) > 0.5  )
        % Some bad offsets, despite no big gaps: must be accumulating bad offsets
        % Interpolate data onto good sample gap
        if ~isa(chdata,'double') & ~isa(chdata,'single'),
            temp=cast(chdata,'double');
            chdata=interp1(chsamps,temp,0:floor(chsamps(end)),'linear');
            chdata=cast(temp,class(chdata));
        else   
            chdata=interp1(chsamps,chdata,0:floor(chsamps(end)),'linear');
        end
        ts_warning('Data had some residual bad offsets, interpolated');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dfile=getFileInfo(dfile)
    % READS metadata (instrument position, sample rate, start/end times, etc into the dfile object
    
    % Programmers: stuff as much information as you can into the dfile object.  This allows
    % Tiskit (or the user) to get information about the file without reading the whole thing
    % If there isn't an appropriate parameter in ts_file, put the value in the "custom"
    % cell array and note what values are will be returned in the custom array in the subroutine
    % comments at the top of this file
    
    [X, Info] = ts_rdmseed(dfile.filename);
    
    % Fill station, network and component info
    for i= 1:length(Info)
%        C=textscan(Info(i).ChannelFullName,'%s:%s:%s:%s');
        C=regexp(Info(i).ChannelFullName,':','split');
        network=C{1};
        station=C{2};
        %location=C{3};
        component=C{4};
        dfile.ch_metadata(i).station=station;
        dfile.ch_metadata(i).network=network;
        dfile.ch_metadata(i).component =component;
        
    end
    
    % Calculate global start and end times
    for i=1:length(X)
        starttimes(i)=ts_datetime(X(i).RecordStartTime);
    end
    lengths=([X.NumberSamples]-1) ./ [X.SampleRate];
    endtimes=starttimes+lengths;
    dfile.privateFirstTime=min(starttimes);
    dfile.privateLastTime=max(endtimes);
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function checkCustom()
%Use this if you have custom values in the .custom parameter
%return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function [startAddress,endAddress]=calcCutAddrs(tdata)
%% This routine will tell you what addresses to keep/read if you only want
%% to read part of the data
%    numsamps = get(tdata,'numsamps');
%    samprate = get(tdata,'samprate');
%
%    if isa(start,'ts_datetime'),
%        timeOffset = start-get(tdata,'chstarttime',1);
%        startAddress = round(timeOffset*samprate +1);
%    elseif start~=0,
%        startAddress = round(start*samprate +1);
%    else
%        startAddress = 1;
%    end
%    endAddress = numsamps-startAddress+1;
%    if  readLen~=0,
%        endAddress = startAddress+(readLen*samprate) - 1;
%    end
%return;
