function writeGSE2_PZ(obj,fname,startDate,refFreq,station,comp,inSPS,outSPS,sensorType,digType,digGain,comments)
    % Write instrument response to a GSE2 Pole-Zero file. (GSE 2.0)
    % Usage:
    %   writeGSE2_PZ(obj,fname,startDate,refFreq,station,comp,inSPS,outSPS,seisName,digName,digGain);
    %   writeGSE2_PZ(obj,fname,);
    %   writeGSE2_PZ(obj,fname);
    %
    %       the filename is generated from the station, component and startDate
    % Inputs:
    %   obj         ts_response object
    %   fname       filename to write to (will use SEISAN naming if empty or absent)
    %   startDate   Date from which these values apply
    %   refFreq     Frequency at which to give the system response (nm/c)
    %   station     Station name
    %   component   Component name (up to three characters)
    %   inSPS       Sample rate at digitizer input
    %   outSPS      Sample rate at digitizer output
    %   sensorType  Sensor type (i.e., 'STS2' 'T240' 'L-28')
    %   digType     Digitizer type (i.e. 'DM-24', 'L-CHEAPO')
    %   digGain     Digitizer gain (counts/V, object's gain will be divided by this
    %                   to calculate the sensor gain)
    %   comments    Comments to put in GSE2.0 file.  If a cell array of
    %                   strings, each string will be put on a different line
    %
    
    %   option      'noConvert' will not convert inUnits to nanometers
    %     NOT IMPLEMENTED?????????
    %
    % The file specifications come from
    % http://www.seismo.ethz.ch/prod/autodrm/manual/CRP243.OpsAnx3.A4.pdf
    %
    % Note that the GSE2.1 format is different enough to be unreadble by a
    % 2.0 reader, but has no tag to distinguish from 2.0!
    % http://www.seismo.ethz.ch/prod/autodrm/manual/provisional_GSE2.1.pdf
    
    if ~exist('fname','var')    fname=''; end
    
    while ~exist('startDate','var')
        f=input('Enter start Date [YYYY/MM/DD HH:MM]: ','s');
        [~,count]=sscanf(f,'%d/%d/%d %d:%d');
        if count==5,
            startDate=ts_datetime(f);
        end
    end
    GSE.startDate=ts_datetime(startDate);
    while ~exist('refFreq','var')
        f=input('Enter reference frequency for system gain: ','s');
        if length(strnum(f))==1;
            refFreq=str2double(f);
        end
    end
    GSE.refFreq=refFreq;
    while ~exist('station','var')
        f=input('Enter station name (up to 4 characters): ','s');
        if length(f)<=4,
            station=(f);
        end
    end
    GSE.station=station;
    while ~exist('comp','var')
        f=input('Enter component name (up to 3 characters): ','s');
        if length(f)<=3,
            comp=(f);
        end
    end
    GSE.component=comp;
    while ~exist('inSPS','var')
        f=input('Enter digitizer input sample rate: ','s');
        if length(strnum(f))==1;
            inSPS=str2double(f);
        end
    end
    GSE.inSPS=inSPS;
    while ~exist('outSPS','var')
        f=input('Enter output sample rate: ','s');
        if length(strnum(f))==1;
            outSPS=str2double(f);
        end
    end
    GSE.outSPS=outSPS;
    while ~exist('sensorType','var')
        f=input('Enter sensor Type (i.e. ''T240'',''L-28''): ','s');
        if length(f)<=8,
            sensorType=(f);
        end
    end
    GSE.sensorType=sensorType;
    while ~exist('digType','var')
        f=input('Enter digitizer Type (i.e. ''DM24'',''LCHEAPO''): ','s');
        if length(f)<=8,
            digType=(f);
        end
    end
    GSE.digType=digType;
    firstTime=1;
    while ~exist('digGain','var')
        if firstTime
            disp('THE SENSOR GAIN WILL BE CALCULATED AS THE OBJECT GAIN DIVIDED BY THE DIGITIZER GAIN');
            firstTime=0;
        end
        f=input('Enter digitizer gain(counts/V): ','s');
        if length(strnum(f))==1;
            digGain=str2double(f);
        end
    end
    GSE.digGain=digGain;
    if ~exist('comments','var')
        comments='';
    end
    GSE.comments=comments;
    
    
    % TEST IF GSE STRUCTURE IS COMPLETE
    fields={'startDate','refFreq','station','component', 'inSPS', ...
        'outSPS','sensorType', 'digType','digGain','comments'};
    tf=isfield(GSE,fields);
    if ~all(tf)
        fieldlist=sprintf('%s ',fields{tf==0});
        error('GSEprops structure is missing fields "%s"',fieldlist);
    end
    
    % CONSTRUCT FILENAME IF NECESSARY
    if isempty(fname)
        fname=[fillSpaces(GSE.station,5) ...
            fillSpaces([GSE.component(1:2) '_' GSE.component(3)],4) ...
            '.' GSE.startDate.char('%y-%m-%d-%H%M_GSE')];
    end
    
    ts_verbose(0,'Writing GSE2 pole-zero file %s\n',fname);
    fid = fopen(fname,'w');
    %% Line 1: Overall
    if ~strcmpi(obj.inUnits,'nm'),
        ts_verbose(0,'Converting inUnits to ''nm''',obj.inUnits);
        obj.inUnits='nm';
    end
    calib = 1/abs(obj.response(GSE.refFreq));  % nm/count
    %abs(obj.response(GSE.refFreq))
    %plot(obj);
    auxID='';
    % GSE 2.0
    fprintf(fid,'CAL2 %-5s %-3s %4s %-6s %10.2e %7.3f %10.5f %s\n', ...
        GSE.station,GSE.component,auxID,GSE.sensorType,calib,1/GSE.refFreq,GSE.outSPS,GSE.startDate.char('%y/%m/%d %H:%M'));
    % GSE 2.1
    %fprintf('CAL2 %-5s %-3s %4s %-6s %10.2e      %7.3f %11.5f %s\n', ...
    %    GSE.station,GSE.component,auxID,GSE.sensorType,calib,1/GSE.refFreq,GSE.outSPS,GSE.startDate.char('%y/%m/%d %H:%M'));
    %% Comments (if exist)
    if ~isempty(GSE.comments)
        if ~iscell(GSE.comments)
            comm={GSE.comments};
        else
            comm=GSE.comments
        end
        for i = 1:length(comm);
            fprintf(fid,' ( %s )\n',strtrim(comm{i}));
        end
    end
    %% Line 2: Seismometer
    sensNormConst=obj.gain/GSE.digGain; %(V/nm)
    nPoles=length(obj.poles);
    nZeros=length(obj.zeros);
    fprintf(fid,'PAZ2 %2d %1s %15.8e %-4s %8.3f %3d %3d %s          \n', ...
        1,'V',sensNormConst,'',0,nPoles,nZeros,'Laplace transform');
    %% Lines 3-N: poles and zeros
    for i=1:length(obj.poles);
        fprintf(fid,' %15.8E %15.8E\n',real(obj.poles(i)),imag(obj.poles(i)));
    end
    for i=1:length(obj.zeros);
        fprintf(fid,' %15.8E %15.8E\n',real(obj.zeros(i)),imag(obj.zeros(i)));
    end
    
    %% Last Line: Digitizer
    fprintf(fid,'DIG2 %2d %15.8e %11.5f %s     \n',2,GSE.digGain,GSE.inSPS,GSE.digType);
    fclose(fid);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function str=fillSpaces(str,numchars)
    % Replaces spaces by underscores and adds underscores to end
    str=strrep(str,' ','_');
    if length(str)>numchars, % String is too long: error
        error('string %s is too long: should be at most %d characters',str,numchars);
    else
        while length(str)<numchars, % String is too short: pad with underscores
            str=[str '_'];
        end
    end
end

