function writeSAC_PZ(obj,fname,convert,norm_freq,inst_gain,inst_gain_inunits)
% Write instrument response to a SAC Pole-Zero file.
%   Converts inUnits to 'nm' unless impossible or option=='noConvert'
% Usage:
%   writeSAC_PZ(obj,fileName,convert,norm_freq,inst_gain);
% Inputs:
%   obj         ts_response object
%	fileName	name of the file to write to.  If not specified,
%               the file will be named 'SAC_PZs_TiSkit'
%   convert     convert inUnits to nanometers: 0 = no, 1=yes (default=1)
%   norm_freq   Frequency (Hz) at which to provide AO (normalisation factor) and
%               SENSITIVITY =CONSTANT/A0 or CONSTANT*1e-9/A0 if in passband)
%   inst_gain   The instrument gain in Volts/INPUT_UNITS at norm_freq
%   INPUT_UNITS 'M/S' by default

if ~exist('fname','var'); fname='SAC_PZs_TiSkit'; end
ts_verbose(0,'Writing SAC pole-zero file %s\n',fname);
convertit=1;
if exist('convert','var')
    if ischar(convert)
        if strcmpi('noconvert',convert),
            convertit=0;
        else
            error('unknown convert string: "%s"',convert)
        end
    else
        convertit=convert;
    end
end
if ~exist('norm_freq','var')
    norm_freq=nan;
end
if ~exist('inst_gain','var')
   inst_gain=nan;
end
if ~exist('inst_gain_inunits','var')
    inst_gain_inunits='M/S';
end
% GET A0 and sensitivity BEFORE CHANGING UNITS
if ~isnan(norm_freq)
    [a0,sensitivity]=obj.getNorm(norm_freq);
    sens_units=upper(obj.inUnits);
end

if convertit,
    if ~strcmpi(obj.inUnits,'nm'),
        ts_verbose(0,'trying to convert inUnits to ''nm''',obj.inUnits);
        obj.inUnits='nm';
    end
end	

fid = fopen(fname,'w');
% SAC comment lines start with '*'
fprintf(fid,'* **********************************\n');
fprintf(fid,'* INPUT UNIT        : %s\n',upper(obj.inUnits));
fprintf(fid,'* OUTPUT UNIT       : %s\n',upper(obj.outUnits));
if ~isnan(inst_gain)
    fprintf(fid,'* INSTGAIN          : %8.6e (%s)\n',inst_gain,upper(inst_gain_inunits));
end
if ~isnan(norm_freq)
    fprintf(fid,'* COMMENT           : SENSITIVITY AND A0 CALCULATED AT %g HZ\n',norm_freq);
    fprintf(fid,'* SENSITIVITY       : %8.6e (%s)\n',sensitivity,sens_units);
    fprintf(fid,'* A0                : %8.6e\n',a0);
end
fprintf(fid,'* **********************************\n');



nP = length(obj.poles); nZ = length(obj.zeros);

if nZ > 0,
	fprintf(fid,'ZEROS %d\n',nZ);
	for i = 1:nZ,
		zer = obj.zeros(i);
		if zer ~= 0,
			fprintf(fid,'%9g %9g\n', real(zer), imag(zer));
		end
	end
end
if nP > 0,
	fprintf(fid,'POLES %d\n',nP);
	for i = 1:nP,
		pole = obj.poles(i);
		if pole ~= 0,
			fprintf(fid,'%9g %9g\n', real(pole), imag(pole));
		end
	end
end
fprintf(fid,'CONSTANT %g',obj.gain);
fclose(fid);
