classdef ts_spectra
    % PRIMARY TISKIT CLASS: Power spectra and coherencies
    %  Objects are created by ts_data.calcSpectra()
    % 
    % ts_spectra properties:
    %  READ AND WRITEABLE
    %    figureprops     - figure properties  (ts_figureproperties)
    %  READ-ONLY, SET IN CONSTRUCTOR
    %     freqs          - spectra and coherency frequencies
    %     spectdata      - spectra data
    %     coherdata      - coherency data
    %     windowsamples  - Spectral window length (samples)
    %     windowtaper    - window taper used to make spectra
    %     cohsigniflevel - coherency 95% significant level
    %     starttime      - Start time of first fft
    %     timeoffsets    - Offsets (seconds) of all ffts from starttime
    %     sinfo          - spectra channel information
    % Dependent (READ-ONLY)
    %     nChannels      - Number of spectra channels
    %     nFreqs         - Number of frequencies in each spectrum
    %     nWindows       - Number of windows used to calculate spectra
    %     starttime      - Time of first data used
    %     endtime        - Time of last data used
    %     windowlength   - Spectral window length (in seconds)
    %
    % ts_spectra methods:
    %   ts_spectra - Constructor
    %   isempty    - Returns 1 if there is no spectra data
    %   plot       - Plot the spectra
    %   plotcoher  - Plot the coherency
    %   calcXF     - Calculate a transfer function from the spectra
    %
    % EXAMPLE:
    %  sps=100; 
    %  t=0:1/sps:200;
    %  data=[sin(5*t)+2*sin(11*t)+0.1*rand(size(t)); ...
    %        sin(5*t)+2*sin(19*t)+0.1*rand(size(t))];
    %  d=ts_data(data,100);
    %  figure(1);plot(d,0,10)
    %  s=d.calcSpectra('wl',15)
    %  figure(2); plot(s)
    %  figure(3); plotcoher(s)
    
    
    %% PUBLIC PROPERTIES (USER CAN CHANGE)
    properties
        figureprops = ts_get('figureprops'); % figure information: also has subplot info
    end
    
    %% PUBLIC PROPERTIES, BUT PRIVATE SET
    properties (SetAccess=private)
        % These are set all at once, don't mess with after
        freqs=[];       % spectra/coherency frequencies
        spectdata=[];   % spectra data
        coherdata=[];   % coherency data
        windowsamples;  % Number of samples in each window used to construct spectrum
        windowtaper=''  % window taper type used in making spectra
        cohsigniflevel=1;   % coherency 95% significancy level (==sqrt(2/nwindows))
        starttime;  % Time of first data used
        timeoffsets;   % vector of offsets from starttime for each fft
        sinfo=ts_spectra_channelinfo;  % spect channel info: .name, .origunits, .units
    end    
    %% DEPENDENT PROPERTIES
    properties (Dependent=true)
        nChannels;   % Number of spectra channels
        nFreqs;   % Number of frequencies in each spectrum
        nWindows; % Number of windows used to calculate spectra
        endtime;    % Time of last data used
        windowlength;   % Length of windows in seconds
    end
    %% PRIVATE PROPERTIES
    properties (Access=private)
        samprate;   % Sampling rate of original data
    end
        
    %% PUBLIC METHODS
    methods
        %% CREATOR
        function obj = ts_spectra(freqs,spectra,coherencies,windsamples,samprate,windtaper,cohsigniflevel,starttime,timeoffsets,chinfo)
            % Create a ts_spectra object
            % Usage:
            %   obj=ts_spectra(freqs,spectra,coherencies,windsamples,samprate,windtaper,cohsigniflevel,starttime,timeoffsets,chinfo)
            
            if nargin==0, 
                return; % Handle empty argument list
            elseif nargin==3,   % Set freq,spectdata and coherdata
                ts_warning('You should set the metadata, too!');
                obj=obj.setdata(freqs,spectra,coherencies);
            elseif nargin==10,   % Set all
                obj=obj.setall(freqs,spectra,coherencies,windsamples,samprate,windtaper,cohsigniflevel,starttime,timeoffsets,chinfo);
            else
                error('Must provide 0, 3 or 10 arguments');
            end
        end % ts_spectra

        %% GET DEPENDENT PROPERTIES
        function M=get.nChannels(obj)
            [M,~]=size(obj.spectdata);
        end
        function M=get.nWindows(obj)
            M=length(obj.timeoffsets);
        end
        function M=get.nFreqs(obj)
            M=length(obj.freqs);
        end
        function out=get.endtime(obj)
            out=obj.starttime + max(obj.timeoffsets)+obj.windowlength;
        end
        function out=get.windowlength(obj)
            out=obj.windowsamples/obj.samprate;
        end

        %% CHARACTER AND DISPLAY METHODS
        function otxt = char(obj,padding)
            %Generate text describing ts_spectra object
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            [M,N]=size(obj.freqs);
            otxt =       sprintf('%s%12s: %dx%d array\n',  pad,'freqs',M,N) ;
            [M,N]=size(obj.spectdata);
            otxt = [otxt sprintf('%s%12s: %dx%d array\n',  pad,'spectdata',M,N)];
            [M,N]=size(obj.coherdata);
            otxt = [otxt sprintf('%s%12s: %dx%d array\n',  pad,'coherdata',M,N)];
            otxt = [otxt sprintf('%s%12s: %-13g\n',        pad,'windowsamples',obj.windowsamples)];
            otxt = [otxt sprintf('%s%12s: %-13g\n',        pad,'nFreqs',obj.nFreqs)];
            otxt = [otxt sprintf('%s%12s: %s\n',           pad,'starttime',obj.starttime.char())];
            otxt = [otxt sprintf('%s%12s: %d %s objects\n',pad,'timeoffsets',length(obj.timeoffsets),class(obj.timeoffsets))];
            otxt = [otxt sprintf('%s%12s: %-13g\n',        pad,'nWindows',obj.nWindows)];
            otxt = [otxt sprintf('%s%12s: %-13g\n',        pad,'cohsigniflevel',obj.cohsigniflevel)];
            otxt = [otxt sprintf('%s%12s: %-13g\n',        pad,'nChannels',obj.nChannels)];
            otxt = [otxt sprintf('\n%s%12s: %s object',    pad,'sinfo',class(obj.sinfo))];
            otxt = [otxt sprintf('\n%s',                   char(obj.sinfo,12+padding))];
            otxt = [otxt sprintf('\n%s%12s: %s object',    pad,'figureprops',class(obj.figureprops))];
            otxt = [otxt sprintf('\n%s',                   char(obj.figureprops,12+padding))];
        end % char  
        function disp(sdata)
             %DISP Display information about ts_spectra object
             disp(char(sdata));
        end %disp
        
        %% OTHER METHODS
        function out=isempty(obj)
            out=isempty(obj.spectdata);
        end  % isempty                    
        [axhand,linehand] = plot(obj,varargin);
        [axhand,linehand] = plotcoher(obj,varargin);
        xfobj = calcXF(obj,varargin);
    end % public methods

    %% PRIVATE METHODS
    methods (Access=private)
        function obj=setall(obj,frqs,sd,cd,windsamples,samprate,windtaper,cohsigniflevel,starttime,timeoffsets,sinfo)
            % Set data
            obj=obj.setdata(frqs,sd,cd);

            if ~isa(windsamples,'numeric') || length(windsamples)~=1,
                error('windsamples must be a numeric scalar');
            end
            obj.windowsamples=windsamples;
            
            if ~isa(samprate,'numeric') || length(samprate)~=1,
                error('samprate must be a numeric scalar');
            end
            obj.samprate=samprate;
            
            if ~isa(windtaper,'char'),
                error('windtaper must be a character string');
            end
            obj.windowtaper=windtaper;

            if ~isa(cohsigniflevel,'numeric') || length(cohsigniflevel)~=1,
                error('cohsigniflevel must be a numeric scalar');
            end
            obj.cohsigniflevel=cohsigniflevel;
            
            if (~isa(starttime,'ts_datetime') || length(starttime) ~=1) && ~isempty(starttime)
                error('timeoffsets must a ts_datetime object');
            end
            obj.starttime=starttime;

            if (~isnumeric(timeoffsets) && ~isempty(timeoffsets))
                error('timeoffsets must a numeric vector');
            end
            obj.timeoffsets=timeoffsets;
            
           if ~isa(sinfo,'ts_spectra_channelinfo') && ~isempty(sinfo)
                error('chinfo must be a vector of ts_spectra_channelinfo objects');
            elseif length(sinfo)~=obj.nChannels,
                error('sinfo has %d elements, but nChannels=%d',length(sinfo),obj.nChannels);
            end
            obj.sinfo=sinfo;
         end
        function obj=setdata(obj,f,s,c)
            % Set the data arrays (freqs, spect, coher)
            if ~isa(f,'numeric'), error('frequency data must be numeric'); end
            if ~isa(s,'numeric'), error('spectra data must be numeric'); end
            if ~isa(c,'numeric'), error('coherency data must be numeric'); end
            [M,N]=size(f);
            if M==1,
                obj.freqs=f;
                nF=N;
            elseif N==1;
                nF=M;
                obj.freqs=f';
            else
                error('frequencies must be a vector');
            end 
            [M,N]=size(s);
            if N==nF, 
                obj.spectdata=s;
            elseif M==nF,
                obj.spectdata=s';
            else
                error('no dimension in spectrum array (%dx%d) matches the # of frequencies (%d)', ...
                        M,N,nF);
            end
            if isempty(c), obj.coherdata=[];
            else
                [M,N,O]=size(c);
                if O==nF,
                    obj.coherdata=c;
                elseif N==nF,
                    error('number of frequencies is in the wrong dimension of the coherence array (2, should be 3)');
                elseif M==nF,
                    error('number of frequencies is in the wrong dimension of the coherence array (1, should be 3)');
                else
                    error('no dimension in coherence array (%dx%dx%d) matches the # of frequencies (%d)', ...
                        M,N,O,nF);
                end
            end
            obj.figureprops.nSubplots=obj.nChannels;
        end % setdata
    end % Private Methods
end % classdef
    