classdef ts_spectra_channelinfo
    % Class for ts_spectra channel information
    %
    % ts_spectra_channelinfo properties:
    %   name      - Spectrum name (station + component)
    %   origunits - Input units of the time series channel
    %   units     - READ ONLY: Units of the power spectral density
    %
    % ts_spectra_channelinfo methods:
    %   Creator
    %      ts_spectra_channelinfo
    %
    % EXAMPLE
    % obj=ts_spectra_channelinfo('BHZ','m/s^2')
    % obj.units
    
    properties
       name='';         % Spectrum name (station+component)
       origunits='';    % Units of time-series channel
    end
    properties(Dependent=true)
        units;          % READ ONLY: Units of power spectral density function
    end
    
    methods
        %% CREATOR
        function obj=ts_spectra_channelinfo(name,origunits)
            % Create a ts_spectra_channelinfo object
            % Usage: obj=ts_spectra_channelinfo();    -or-
            %        obj=ts_spectra_channelinfo(name);    -or-
            %        obj=ts_spectra_channelinfo(name,origunits)
            if nargin==0, return; end
            obj.name=name;
            if exist('origunits','var')
                obj.origunits=origunits;
            end
        end
        
        %% SET METHODS
        function obj=set.name(obj,val)
            if ~isa(val,'char'), error('value must be a character string'); end
            obj.name=val;
        end
        function obj=set.origunits(obj,val)
            if ~isa(val,'char'), error('value must be a character string'); end
            obj.origunits=val;
        end
        
        %% GET METHODS
        function out=get.units(obj)
            out=sprintf('(%s)^2/Hz',obj.origunits);
        end
         
        %% DISPLAY AND CHAR METHODS
        function txt=char(obj,padding)
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            txt=[];
            tmp=[]; for i=1:length(obj), ttxt=sprintf('%-13s',['''' obj(i).name '''']); tmp=[tmp ttxt(1:13)]; end
            txt=[txt sprintf('%s%10s:        %s\n',pad,'name',tmp)];
            tmp=[]; for i=1:length(obj), ttxt=sprintf('%-13s',['''' obj(i).origunits '''']); tmp=[tmp ttxt(1:13)]; end
            txt=[txt sprintf('%s%10s:        %s\n',pad,'origunits',tmp)];
            tmp=[]; for i=1:length(obj), ttxt=sprintf('%-13s',['''' obj(i).units '''']); tmp=[tmp ttxt(1:13)]; end
            txt=[txt sprintf('%s%10s:        %s',pad,'units',tmp)];
         end        
       function disp(obj)
           disp(char(obj));
       end     

    end %methods
end % classdef
        