classdef ts_station
    % Holds information about a single station.
    % For details, type: 'doc ts_station'
    properties
        name='';        % Station name
        network='';     % 2-lettter code for seismology network
        location='00';  % Used to specify different instruments within a seismology stations
        information=''; % Can hold instrument types, serial numbers, etc
        timezero=ts_datetime(); % GMT time at which station was synchronized
                        % Used to account for synchronization offsets and/or a non-GMT timezone
        lat=0;          % Decimal degrees N
        lon=0;          % Decimal degrees E
        elev=0;         % Instrument elevation above sealevel (m)
        wdepth=0;       % Water depth at station (0 if land station, -elev if station at sea floor)
        tzoffset=0;     % offset (inst-GMT, seconds) of the clock at synchronization
        clockdrift=0;	% clock drift == (endTime(inst)-endTime(GPS) ) / (endTime-startTime)
    end % properties
    
    methods
        %% CONSTRUCTOR
        function obj=ts_station(name,network,information,timezero,lat,lon,elev,wdepth)
            % Constructor
            % Usage=obj=ts_station(name,network,information,timezero,lat,lon,elev,wdepth)
            % You can stop where you want, the rest will take their default
            % values
            ts_init()
            if nargin==0, return; end
            obj.name=name;
            if nargin==1, return; end
            obj.network=network;
            if nargin==2, return; end
            obj.information=information;
            if nargin==3, return; end
            obj.timezero=timezero;
            if nargin==4, return; end
            obj.lat=lat;
            if nargin==5, return; end
            obj.lon=lon;
            if nargin==6, return; end
            obj.elev=elev;
            if nargin==7, return; end
            obj.wdepth=wdepth;
        end % ts_station
        
        %% SET functions (check ranges)
        function obj = set.name(obj,value)
            if ischar(value), obj.name=value;
            else error('name must be a character string');
            end
        end
        function obj = set.network(obj,value)
            if ischar(value), obj.network=value;
            else error('network must be a character string');
            end
        end
        function obj = set.location(obj,value)
            if (ischar(value) && length(value)==2), obj.location=value;
            else error('network must be a 2-character string');
            end
        end
        function obj = set.information(obj,value)
            if ischar(value), obj.information=value;
            else error('information must be a character string');
            end
        end
        function obj = set.timezero(obj,value)
            if ischar(value)
                obj.timezero=ts_datetime(value);
            elseif isa(value,'ts_datetime'),
                obj.timezero=value;
            else
                error('timezero must be a character string or ts_datetime object');
            end
            if obj.timezero==ts_datetime(); ts_warning('You set timezero to null value!'); end
        end
        function obj = set.tzoffset(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.tzoffset=value;
                else error('tzoffset must be a scalar value');
                end
            else error('tzoffset must be a scalar numerical value');
            end
        end
        function obj = set.clockdrift(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.clockdrift=value;
                else error('clockdrift must be a scalar value');
                end
            else error('clockdrift must be a scalar numerical value');
            end
        end
        function obj = set.lat(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    if value >= -90 && value <= 90
                        obj.lat=value;
                    else error('lat must be between -90 and 90');
                    end
                else error('lat must be a scalar value');
                end
            else error('lat must be a scalar numerical value');
            end
        end
        function obj = set.lon(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    if value >= -360 && value <= 360,
                        obj.lon=value;
                    else error('lon must be between -360 and 360');
                    end
                else error('lon must be a scalar value');
                end
            else error('lon must be a scalar numerical value');
            end
        end
        function obj = set.elev(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.elev=value;
                else error('elev must be a scalar value');
                end
            else error('elev must be a scalar numerical value');
            end
        end
        function obj = set.wdepth(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.wdepth=value;
                else error('wdepth must be a scalar value');
                end
            else error('wdepth must be a scalar numerical value');
            end
        end
        
		%% EQUALITY and INEQUALITY
		result=eq(a,b)
			% EQ returns 1 if name,location, & network are identical
			result=~neq(a,b);
		end
		
		result=neq(a,b)
			% NEQ returns 1 if any of station,location & network are not identical
			if ~strcmp(a.name,b.name); result=1; return; end
			if ~strcmp(a.location,b.location); result=1; return; end
			if ~strcmp(a.network,b.network); result=1; return; end
			result=0;
		end
        %% TEXT DISPLAY
        txt = char(obj,fmt);       % DEFINED IN SEPARATE FILE
        function disp(obj)
            % Displays object information when not followed by a semi-colon
            disp(char(obj));
        end
        
        % Removed setdisp because it is only useful for hgetset
        % superclasses, and we don't want stations to be handles!!!!
        %         %% REDEFINE SETDISP TO SPECIFY INPUTS
        %         function setdisp(obj)
        %             pnames=properties(obj);
        %             for i=1:length(pnames),
        %                 p=pnames{i};
        %                 v=get(obj,p);
        %                 switch p,
        %                     case 'lat', vstr='scalar between -90 and 90';
        %                     case 'lon', vstr='scalar between -360 and 360';
        %                     case 'timezero', vstr='ts_datetime object';
        %                     case {'name','network','information'},
        %                         vstr='character string';
        %                     case {'tzoffset','clockdrift','elev'},
        %                         vstr='scalar value';
        %                     case 'wdepth', vstr='non-negative scalar value';
        %                     otherwise, vstr='';
        %                 end
        %                 fprintf('%15s:  %s\n',p,vstr);
        %             end
        %         end
        
        function outTime = GPSTime(obj,instTime)
            % Convert instrument time to GPS time
            % Usage: GPSTime=GPSTime(obj,instTime) -or-
            %       GPSTime=obj.GPSTime(instTime)
            %
            %   where instTime is the instrument time
            %
            % Corrects for time-zero offset and clock drift, as stored in
            % ibj and defined by:
            %   obj.clockdrift	= (instElapsed - GPSElapsed)/GPSElapsed
            %   obj.timezero    = GPS time at which the instrument was first synchronized
            %   obj.tzoffset    = instTime - GPSTime (seconds) at timezero (usually 0)
            outTime=obj.driftCorrect(instTime,'forward');
        end
        
        function outTime = instrumentTime(obj,GPStime)
            % Convert GPS time to instrument time
            % Usage: instTime=instrumentTime(obj,GPSTime)
            %
            %   where GPStime is the true (GPS) time
            %
            %   used when searching for data in an instrument file
            %
            % Corrects for time-zero offset and clock drift, as stored in
            % the object and defined by:
            %   clockdrift	= (instElapsed - GPSElapsed)/GPSElapsed
            %   timezero    = GPS time at which the instrument was first synchronized
            %   tzoffset    = instTime - GPSTime (seconds) at timezero
            outTime=obj.driftCorrect(GPStime,'backward');
        end
        
    end % Public methods
    
    methods (Access = private)
        function outTime = driftCorrect(obj,mytime,option)
            % Correct an instrument time for (linear) clock drift
            % Usage: correctTime = driftCorrect(obj,mytime);
            %     -or-  instTime = driftCorrect(obj,mytime,'GPStoInst');
            %
            %   Uses the 'timezero', 'tzoffset' and 'clockdrift' parameters
            %       in the ts_file object, where:
            %       clockdrift	= (instElapsed - GPSElapsed)/GPSElapsed
            %       timezero	is the (GPS) time at which the instrument was
            %                       synchronized to GPS
            %       tzoffset    is the offset in seconds of the instrument from GPS
            %                   time at timezero (==instTime-timezero @ timezero)
            %
            % Input:
            %   mytime is the time given by the instrument clock (ts_datetime object)
            %       or a time interval (in seconds) [timezero and tzoffset are unused in this case]
            % Output:
            %	ts_datetime object
            %
            % Equations used:
            %   insttime= GPStime  + tzoffset + GPSElapsed*clockdrift
            %   GPStime = insttime - tzoffset - instElapsed*clockdrift/(1+clockdrift)
            %
            %  The first form gives the GPS time corresponding to an instrument time
            %	(useful for correcting instrument times in printouts)
            %  the second form gives the instrument time corresponding to a GPS time
            %	(useful for choosing an instrument data sample corresponding to a GPS time)
            
            ts_debug(2,'Starting...');
            if ~exist('option','var'), option='forward'; end
            clockdrift=obj.clockdrift;
            tzoffset=obj.tzoffset;
            timezero=obj.timezero;
            switch lower(option),
                case {'insttogps','forward','forwards'},    % Convert from instrument to GPS
                    if isa(mytime,'ts_datetime'),
                        if clockdrift==0,
                            outTime=mytime-tzoffset; % avoids calculating mytime-timezero, which can be long if timezero not set
                        else
                            if timezero==ts_datetime();  % Time-zero was never set, don't correct
                                error('You set a clock drift, but not time zero!');
                            end
                            instElapsed=mytime-(timezero+tzoffset);	% Seconds since sync
                            outTime=mytime - tzoffset - instElapsed*clockdrift/(1+clockdrift);
                        end
                    else
                        ts_verbose(2,'"mytime" is not a ts_datetime object, assuming it is a time offset');
                        %outTime=mytime/(1+clockdrift);      % True solution
                        outTime=mytime - mytime*clockdrift;  % First order solution, might be more numerically stable
                    end
                case {'gpstoinst','backward','backwards'},  % Convert from GPS to instrument
                    if isa(mytime,'ts_datetime'),
                        if clockdrift==0,
                            outTime=mytime+tzoffset;     % avoids calculating mytime-timezero, which can be long if timezero not set
                        else
                            if timezero==ts_datetime();  % Time-zero was never set, don't correct
                                error('You set a clock drift, but not time zero!');
                            end
                            GPSElapsed=mytime-timezero;				% Seconds since sync
                            outTime=mytime + tzoffset + GPSElapsed*clockdrift;
                        end
                    else
                        ts_verbose(1,'"mytime" is not a ts_datetime object, assuming it is a time offset');
                        GPSElapsed=mytime;
                        outTime=GPSElapsed+GPSElapsed*clockdrift;
                    end
                otherwise,
                    error(['Unknown option: ' option]);
            end
            ts_debug(2,'Ending');
        end % private method driftCorrect
    end % Private methods
end % classdef
